/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: hwsimso0.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
/* Zend API library			*/
#include "php.h"					/* PHP library				*/
/* IMS extension libraries	*/
#include "connect.h"				/* IMS Connector			*/
#include "hwsimso0.h"				/* Simple module : HWSIMSO0	*/


/**
 * SIMPLE Module Exit configuration (used by manager)
 */
int	HWSIMSO0_setModule(IMS_factory_t *factory)
{
	/* Module data allocation for IMS message		*/
	
	if ((factory->message->modConfig = (HWSIMSO0_module_t *) emalloc(sizeof(HWSIMSO0_module_t))) == NULL)
		return IMS_CODE_ERR_MALLOC;
	
	/* Module functions bindings to Exit Manager	*/

	factory->build	= (IMS_build_f)	&HWSIMSO0_buildMessage;
	factory->write	= (IMS_write_f)	&HWSIMSO0_writeMessage;
	factory->read	= (IMS_read_f)	&HWSIMSO0_readMessage;

	return IMS_CODE_OK;
}

/**
 * IMS message total length getter
 */
int HWSIMSO0_getMsgLength(IMS_message_t *message)
{
	/* Sum of lengths :	Message length field	(4 bytes)
	 *				  +	Message header			(HDR_IRM_LEN)
	 *				  + Message record data		(msgRecordLen)
	 *				  +	Final LLZZ tag			(4 bytes)
	 */
	return TOTAL_LENGTH_LEN + HWSIMSO0_HDR_IRM_LEN + message->msgRecordLen + LLZZ_LEN;
}

/**
 * IMS message setter (Simple Exit module)
 */
int HWSIMSO0_buildMessage(IMS_message_t *message, IMS_connect_t *params)
{
	IMS_prefix_t *		prefix	= message->imsPrefix;
	HWSIMSO0_module_t *	module	= (HWSIMSO0_module_t *)message->modConfig;
	HWSIMSO0_header_t * hdrIRM	= &(module->headerIRM);
	
	/* IMS message total length			*/
	
	message->msgLength		= HWSIMSO0_getMsgLength(message);
	
	/* IMS prefix header parameters		*/
	
	prefix->irmLen			= HWSIMSO0_HDR_IRM_LEN;
	prefix->exitId			= IMS_makeField(HWSIMSO0_IRM_ID, strlen(HWSIMSO0_IRM_ID), HEADER_STR_LEN, IMS_BLANK);
	prefix->reserved		= IRM_F5_NONE;
	prefix->msgType			= 0x00;
	prefix->timer			= 0x00;
	prefix->socket			= 0x00;
	prefix->clientId		= IMS_makeField(params->clientId, params->clientIdLen, HEADER_STR_LEN, IMS_BLANK);
	
	/* IMS module headers parameters	*/
	
	hdrIRM->MFS_mod			= MFS_MOD_OFF;
	hdrIRM->commitMode		= COMMIT_MODE_ON;
	hdrIRM->syncLevel		= SYNC_LEVEL_NONE;
	hdrIRM->nad				= SYNC_FLAG_BLANK;
	hdrIRM->tranCode		= IMS_makeField(params->tranCode, params->tranCodeLen, HEADER_STR_LEN, IMS_BLANK);
	hdrIRM->datastoreId		= IMS_makeField(params->datastoreId, params->datastoreIdLen, HEADER_STR_LEN, IMS_BLANK);
	hdrIRM->lterm			= IMS_makeField(params->lterm, params->ltermLen, HEADER_STR_LEN, IMS_BLANK);
	hdrIRM->racfUserId		= IMS_makeField(params->racfUserId, params->racfUserIdLen, HEADER_STR_LEN, IMS_BLANK);
	hdrIRM->racfGroupName	= IMS_makeField(params->racfGroupName, params->racfGroupNameLen, HEADER_STR_LEN, IMS_BLANK);
	hdrIRM->racfPassword	= IMS_makeField(params->racfPassword, params->racfPasswordLen, HEADER_STR_LEN, IMS_BLANK);
	
	return IMS_CODE_OK;
}

/**
 * IMS message writer
 */
int HWSIMSO0_writeMessage(IMS_message_t *message, binaryDataBuffer_t *dataBuffer)
{
	int	written = 0;
	
	/* Message total length field	*/
	written += binaryDataBuffer_writeInt(dataBuffer, message->msgLength);
	/* Message header areas			*/
	written += HWSIMSO0_writeHeaders(dataBuffer, message);
	/* Message record data			*/
	if (message->msgRecord)
		written += IMS_writeRecord(dataBuffer, message);
	/* Message final LLZZ tag		*/
	written += IMS_writeTagLLZZ(dataBuffer, message, LL_TAG_MSG_END);
	
	return written;
}

/**
 * Module header areas writer
 */
int HWSIMSO0_writeHeaders(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	int	written = 0;
	
	/* IRM prefix header	*/
	written += IMS_writePrefix(dataBuffer, message);
	/* User prefix header	*/
	written += HWSIMSO0_writeUsrPrefix(dataBuffer, message);
	
	return written;
}

/**
 * User prefix header writer
 */
int HWSIMSO0_writeUsrPrefix(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	HWSIMSO0_module_t *	module	= (HWSIMSO0_module_t *)message->modConfig;
	HWSIMSO0_header_t *	hdrIRM	= &(module->headerIRM);
	int					written	= 0;

	/* IRM_F1 field				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrIRM->MFS_mod);
	/* IRM_F2 field				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrIRM->commitMode);
	/* IRM_F3 field				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrIRM->syncLevel);
	/* IRM_F4	 field			*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrIRM->nad);
	/* IRM_TRNCOD field			*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrIRM->tranCode, HEADER_STR_LEN, message->encoder);
	/* IRM_IMSDESTID field		*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrIRM->datastoreId, HEADER_STR_LEN, message->encoder);
	/* IRM_LTERM field			*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrIRM->lterm, HEADER_STR_LEN, message->encoder);
	/* IRM_RACF_USERID field	*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrIRM->racfUserId, HEADER_STR_LEN, message->encoder);
	/* IRM_RACF_GRNME field		*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrIRM->racfGroupName, HEADER_STR_LEN, message->encoder);
	/* IRM_RACF_PW field		*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrIRM->racfPassword, HEADER_STR_LEN, message->encoder);
	
	return written;
}

/**
 * IMS message reader
 */
int HWSIMSO0_readMessage(IMS_message_t *message, binaryDataBuffer_t *dataBuffer)
{
	/* Header total length						*/
	/* IRM prefix = 12 bytes long at reception	*/
	int	headerLength	= 12;
	/* Direct access to offset					*/
	binaryDataBuffer_setOffset(dataBuffer, headerLength);
	/* IMS message length						*/
	message->msgRecordLen	= message->msgLength - headerLength;
	
	return IMS_CODE_OK;
}
