/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: hwsjava0.c,v 1.1 2007/03/27 14:32:36 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
/* Zend API library			*/
#include "php.h"					/* PHP library				*/
/* IMS extension libraries	*/
#include "connect.h"				/* IMS Connector 			*/
#include "hwsjava0.h"				/* Java module :   HWSJAVA0	*/


/**
 * JAVA Module Exit configuration (used by manager)
 */
int	HWSJAVA0_setModule(IMS_factory_t *factory)
{
	/* Module data allocation for IMS message		*/
	
	if ((factory->message->modConfig = (HWSJAVA0_module_t *) emalloc(sizeof(HWSJAVA0_module_t))) == NULL)
		return IMS_CODE_ERR_MALLOC;
	
	/* Module functions bindings to Exit Manager	*/
	
	factory->build	= (IMS_build_f)	&HWSJAVA0_buildMessage;
	factory->write	= (IMS_write_f)	&HWSJAVA0_writeMessage;
	factory->read	= (IMS_read_f)	&HWSJAVA0_readMessage;
	
	return IMS_CODE_OK;
}

/**
 * IMS message total length getter
 */
int HWSJAVA0_getMsgLength(IMS_message_t *message)
{
	/* Sum of message lengths :
	 * 		Length field			(4 bytes)
	 *	+	IRM prefix header		(HDR_IRM_LEN)
	 *	+	MCI header				(HDR_MCI_LEN)
	 *	+	SDF header				(HDR_SDF_LEN)
	 *	+	SEC header				(HDR_SEC_LEN)
	 *	+	USR header				(HDR_USR_LEN)
	 *	+	Message record data		(msgRecordLen)
	 */
	int	msgLengthgth	= 0;
	
	msgLengthgth += TOTAL_LENGTH_LEN;
	msgLengthgth += HWSJAVA0_HDR_IRM_LEN;
	msgLengthgth += HWSJAVA0_HDR_MCI_LEN;
	msgLengthgth += HWSJAVA0_HDR_SDF_LEN;
	msgLengthgth += HWSJAVA0_HDR_SEC_LEN;
	msgLengthgth += HWSJAVA0_HDR_USR_LEN;
	msgLengthgth += message->msgRecordLen;
	
	return msgLengthgth;
}

/**
 * IMS message setter (Java Exit module)
 */
int HWSJAVA0_buildMessage(IMS_message_t *message, IMS_connect_t *params)
{
	IMS_prefix_t *			prefix	= message->imsPrefix;
	HWSJAVA0_module_t *		module	= (HWSJAVA0_module_t *) message->modConfig;
	HWSJAVA0_headerMCI_t *	hdrMCI	= &(module->headerMCI);
	HWSJAVA0_headerSDF_t *	hdrSDF	= &(module->headerSDF);
	HWSJAVA0_headerSEC_t *	hdrSEC	= &(module->headerSEC);
	HWSJAVA0_headerUSR_t *	hdrUSR	= &(module->headerUSR);
	
	/* IMS message total length			*/

	message->msgLength	= HWSJAVA0_getMsgLength(message);

	/* IMS prefix header parameters		*/
	
	prefix->irmLen		= HWSJAVA0_HDR_IRM_LEN;
	prefix->exitId		= IMS_makeField(HWSJAVA0_IRM_ID, strlen(HWSJAVA0_IRM_ID), HEADER_STR_LEN, IMS_BLANK);
	prefix->reserved	= 0x00;
	prefix->msgType		= IRM_F5_OTMA_HEADERS | IRM_F5_CLIENT_TRANS;
	prefix->timer		= 0x00;
	prefix->socket		= 0x00;
	prefix->clientId	= IMS_makeField(params->clientId, params->clientIdLen, HEADER_STR_LEN, IMS_BLANK);

	/* MCI header area parameters : Message Control Infos						*/

	hdrMCI->archLevel		= 0x01;
	hdrMCI->msgType			= 0x40;				/* Arbitrary value				*/
	hdrMCI->responseFlag	= 0x00;
	hdrMCI->commitFlag		= 0x00;
	hdrMCI->cmdType			= 0x00;
	hdrMCI->processFlag		= 0x00;
/*	Not in use !
	hdrMCI->tPipeName		= IMS_makeField(NULL, 0, HWSJAVA0_OMCTLTNM_LEN, IMS_BLANK);
*/
	hdrMCI->tPipeName		= IMS_makeField(params->lterm, params->ltermLen, HWSJAVA0_OMCTLTNM_LEN, IMS_BLANK);
	hdrMCI->chainFlag		= 0xA0;				/* First & Last	: x80 & x20		*/
	hdrMCI->prefixFlag		= 0xF0;				/* All : x80 & x40& x20 & x10	*/
	hdrMCI->sendSeqNb		= 0x00;
	hdrMCI->senseCode		= 0x00;
	hdrMCI->reasonCode		= 0x00;
	hdrMCI->recoverSeqNb	= 0x00;
	hdrMCI->segmentSeqNb	= 0x01;				/* Number of segments : 1		*/
	hdrMCI->reserved		= 0x00;

	/* SDF header area parameters : State Data Format							*/
	
	hdrSDF->length			= HWSJAVA0_HDR_SDF_LEN;
	hdrSDF->serverState		= 0x00;
	hdrSDF->syncFlag		= 0x20;					/* Commit Mode 1			*/
	hdrSDF->syncLevel		= 0x00;
	hdrSDF->clientFlags		= 0x40;
	/* CAUTION :	Map Name hard coded in macro !	*/
	hdrSDF->mapName			= IMS_makeField(HWSJAVA0_MAP_NAME, strlen(HWSJAVA0_MAP_NAME), HWSJAVA0_OMCTLMAP_LEN, IMS_BLANK);
	hdrSDF->serverToken		= IMS_makeField(NULL, 0, HWSJAVA0_OMHDRTOK_LEN, 0x00);
	hdrSDF->correlToken		= IMS_makeField(NULL, 0, HWSJAVA0_OMHDRCOR_LEN, 0x00);
	hdrSDF->contextId		= IMS_makeField(NULL, 0, HWSJAVA0_OMHDRCOR_LEN, 0x00);
	hdrSDF->dstOverride		= IMS_makeField(params->lterm, params->ltermLen, HWSJAVA0_OMHDRLTM_LEN, IMS_BLANK);
	hdrSDF->srvUsrDataLen	= 0x00;					/* Not used					*/
	hdrSDF->srvUsrData		= NULL;					/* Not used					*/
	
	/* SEC header area parameters : Security Data								*/
	
	hdrSEC->length			= HWSJAVA0_HDR_SEC_LEN;
	/* Security Flag -> straight encoded character	*/
	hdrSEC->securityFlag	= IMS_charEncode(HWSJAVA0_SEC_FLAG_FULL, message->encoder);
	hdrSEC->reserved		= 0x14;					/* Arbitrary value			*/
	/* User Token -> Not used						*/
	hdrSEC->uTokenLen		= HWSJAVA0_OMSECRTY_LEN + HWSJAVA0_OMSECPRF_LEN;
	hdrSEC->uTokenType		= HWSJAVA0_SEC_TYPE_TOKEN;
	hdrSEC->uToken			= IMS_makeField(NULL, 0, HWSJAVA0_OMSECPRF_LEN, 0x00);
	/* User ID -> connection parameter				*/
	hdrSEC->userIdLen		= HWSJAVA0_OMSECUTY_LEN + HWSJAVA0_OMSECUID_LEN;			
	hdrSEC->userIdType		= HWSJAVA0_SEC_TYPE_USRID;
	hdrSEC->userId			= IMS_makeField(params->racfUserId, params->racfUserIdLen, HWSJAVA0_OMSECUID_LEN, IMS_BLANK);
	/* Groupe Name -> connection parameter			*/
	hdrSEC->groupeLen		= HWSJAVA0_OMSECGTY_LEN + HWSJAVA0_OMSECGRP_LEN;
	hdrSEC->groupeType		= HWSJAVA0_SEC_TYPE_GROUP;
	hdrSEC->groupe			= IMS_makeField(params->racfGroupName, params->racfGroupNameLen, HWSJAVA0_OMSECGRP_LEN, IMS_BLANK);
	/* Header final filling							*/
	hdrSEC->filler			= IMS_makeField(NULL, 0, HWSJAVA0_HDR_SEC_FILLER, 0x00);	

	/* USR header area parameters : User Data										*/
	
	hdrUSR->length			= HWSJAVA0_HDR_USR_LEN;
	hdrUSR->fullWord		= 0x0000;
	hdrUSR->destId			= IMS_makeField(params->datastoreId, params->datastoreIdLen, HEADER_STR_LEN, IMS_BLANK);
	/* CAUTION :	Origin ID hard coded in macro !	*/
	hdrUSR->origId			= IMS_makeField(params->clientId, params->clientIdLen, HEADER_STR_LEN, IMS_BLANK);
	hdrUSR->portId			= IMS_makeField(NULL, 0, HEADER_STR_LEN, 0x00);
	hdrUSR->lToken			= IMS_makeField(NULL, 0, HEADER_STR_LEN, 0x00);
	hdrUSR->retCode			= 0x00;
	hdrUSR->resCode			= IMS_makeField(NULL, 0, HEADER_STR_LEN, 0x00);
	hdrUSR->rToken			= 0x00;
	hdrUSR->passwd			= IMS_makeField(NULL, 0, HEADER_STR_LEN, 0x00);
	hdrUSR->socket			= HWSJAVA0_OMUSR_PSOCKET;
	hdrUSR->pwdType			= 0x00;
	hdrUSR->origSync		= 0x00;
	hdrUSR->timer			= 0x00;
	/* Header final filling							*/
	hdrUSR->filler			= IMS_makeField(NULL, 0, HWSJAVA0_HDR_USR_FILLER, 0x00);

	return IMS_CODE_OK;
}

/**
 * IMS message writer
 */
int HWSJAVA0_writeMessage(IMS_message_t *message, binaryDataBuffer_t *dataBuffer)
{
	int	written = 0;
	
	/* Message total length field	*/
	written += binaryDataBuffer_writeInt(dataBuffer, message->msgLength);
	/* Message header areas			*/
	written += HWSJAVA0_writeHeaders(dataBuffer, message);
	/* Message record data			*/
	if (message->msgRecord)
		written += IMS_writeRecord(dataBuffer, message);
	
	return written;
}

/**
 * Module headers areas writer
 */
int HWSJAVA0_writeHeaders(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	int	written = 0;

	/* IRM prefix header	*/
	written += IMS_writePrefix(dataBuffer, message);
	/* MCI header area :	Message Control Infos	*/
	written += HWSJAVA0_writeHeaderMCI(dataBuffer, message);
	/* SDF header area :	State Data Format		*/
	written += HWSJAVA0_writeHeaderSDF(dataBuffer, message);
	/* SEC header area :	Security Data			*/
	written += HWSJAVA0_writeHeaderSEC(dataBuffer, message);
	/* USR header area :	User Data				*/
	written += HWSJAVA0_writeHeaderUSR(dataBuffer, message);
	
	return written;
}

/**
 * MCI header area writer : Message Control Infos
 */
int HWSJAVA0_writeHeaderMCI(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	HWSJAVA0_module_t *		module	= (HWSJAVA0_module_t *)message->modConfig;
	HWSJAVA0_headerMCI_t *	hdrMCI	= &(module->headerMCI);
	int						written	= 0;
	
	/* Architecture Level		*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->archLevel);
	/* Message Type				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->msgType);
	/* Response Flag			*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->responseFlag);
	/* Commit Confirmation Flag	*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->commitFlag);
	/* Command Type				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->cmdType);
	/* Process Flag				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->processFlag);
	/* Tpipe Name				*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrMCI->tPipeName, HWSJAVA0_OMCTLTNM_LEN, message->encoder);
	/* Chain Flag				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->chainFlag);
	/* Prefix Flag				*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrMCI->prefixFlag);
	/* Send Sequence Number		*/
	written += binaryDataBuffer_writeInt(dataBuffer, hdrMCI->sendSeqNb);
	/* Sense Code				*/
	written += binaryDataBuffer_writeShort(dataBuffer, hdrMCI->senseCode);
	/* Reason Code				*/
	written += binaryDataBuffer_writeShort(dataBuffer, hdrMCI->reasonCode);
	/* Recoverable Sequence Nbr	*/
	written += binaryDataBuffer_writeInt(dataBuffer, hdrMCI->recoverSeqNb);
	/* Segment Sequence Number	*/
	written += binaryDataBuffer_writeShort(dataBuffer, hdrMCI->segmentSeqNb);
	/* Reserved					*/
	written += binaryDataBuffer_writeShort(dataBuffer, hdrMCI->reserved);
	
	return written;
}

/**
 * SDF header area writer : State Data Format
 */
int HWSJAVA0_writeHeaderSDF(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	HWSJAVA0_module_t *		module	= (HWSJAVA0_module_t *)message->modConfig;
	HWSJAVA0_headerSDF_t *	hdrSDF	= &(module->headerSDF);
	int						written	= 0;
	
	written += binaryDataBuffer_writeShort(dataBuffer, hdrSDF->length);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSDF->serverState);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSDF->syncFlag);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSDF->syncLevel);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSDF->clientFlags);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSDF->mapName, HWSJAVA0_OMCTLMAP_LEN, message->encoder);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSDF->serverToken, HWSJAVA0_OMHDRTOK_LEN, message->encoder);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSDF->correlToken, HWSJAVA0_OMHDRCOR_LEN, message->encoder);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSDF->contextId, HWSJAVA0_OMHDRCID_LEN, message->encoder);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSDF->dstOverride, HWSJAVA0_OMHDRLTM_LEN, message->encoder);
	/* Server User Data : not used	*/
	written += binaryDataBuffer_writeShort(dataBuffer, hdrSDF->srvUsrDataLen);

	return written;
}

/**
 * SEC header area writer : Security Data
 */
int HWSJAVA0_writeHeaderSEC(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	HWSJAVA0_module_t *		module	= (HWSJAVA0_module_t *)message->modConfig;
	HWSJAVA0_headerSEC_t *	hdrSEC	= &(module->headerSEC);
	int						written	= 0;
	
	written += binaryDataBuffer_writeShort(dataBuffer, hdrSEC->length);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->securityFlag);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->reserved);
	/* User Token	*/
	/* Not used
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->uTokenLen);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->uTokenType);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSEC->uToken, HWSJAVA0_OMSECPRF_LEN, message->encoder);
	*/
	/* User ID		*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->userIdLen);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->userIdType);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSEC->userId, HWSJAVA0_OMSECUID_LEN, message->encoder);
	/* Groupe Name	*/
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->groupeLen);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrSEC->groupeType);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSEC->groupe, HWSJAVA0_OMSECGRP_LEN, message->encoder);
	/* Filling	*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrSEC->filler, HWSJAVA0_HDR_SEC_FILLER, message->encoder);

	return written;
}

/**
 * USR header area writer : User Data
 */
int HWSJAVA0_writeHeaderUSR(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	HWSJAVA0_module_t *		module	= (HWSJAVA0_module_t *)message->modConfig;
	HWSJAVA0_headerUSR_t *	hdrUSR	= &(module->headerUSR);
	int						written	= 0;
	
	written += binaryDataBuffer_writeShort(dataBuffer, hdrUSR->length);
	written += binaryDataBuffer_writeShort(dataBuffer, hdrUSR->fullWord);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrUSR->destId, HEADER_STR_LEN, message->encoder);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrUSR->origId, HEADER_STR_LEN, message->encoder);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrUSR->portId, HEADER_STR_LEN, message->encoder);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrUSR->lToken, HEADER_STR_LEN, message->encoder);
	written += binaryDataBuffer_writeInt(dataBuffer, hdrUSR->retCode);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrUSR->resCode, HEADER_STR_LEN, message->encoder);
	written += binaryDataBuffer_writeInt(dataBuffer, hdrUSR->rToken);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrUSR->passwd, HEADER_STR_LEN, message->encoder);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrUSR->socket);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrUSR->pwdType);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrUSR->origSync);
	written += binaryDataBuffer_writeByte(dataBuffer, hdrUSR->timer);
	written += binaryDataBuffer_encodeBytes(dataBuffer, hdrUSR->filler, HWSJAVA0_HDR_USR_FILLER, message->encoder);

	return written;
}

/**
 * IMS message reader
 */
int HWSJAVA0_readMessage(IMS_message_t *message, binaryDataBuffer_t *dataBuffer)
{
	/* Header total length	*/
	/* IRM prefix = 12 bytes long at reception	*/
	int	headerLength	= 12					\
						+ HWSJAVA0_HDR_MCI_LEN	\
						+ HWSJAVA0_HDR_SDF_LEN	\
						+ HWSJAVA0_HDR_SEC_LEN	\
						+ HWSJAVA0_HDR_USR_LEN;
	/* Message length validation	*/
	if (headerLength > message->msgLength) {
		return IMS_CODE_ERR_READING;
	}
	/* Direct access to offset					*/
	binaryDataBuffer_setOffset(dataBuffer, headerLength);
	/* IMS message record data					*/
	message->msgRecord		= binaryDataBuffer_getBuffer(dataBuffer);
	/* IMS message record length				*/
	message->msgRecordLen	= message->msgLength - headerLength;
	
	return IMS_CODE_OK;
}
