/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: hwsjava0.h,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


#ifndef	_IMS_HWSJAVA0_H_
#define	_IMS_HWSJAVA0_H_

/*********************************************************************
 * Constants definitions
 */

/* IRM header prefix	*/

#define HWSJAVA0_HDR_IRM_LEN		0x001C		/* IRM Header :			28 bytes	*/

#define HWSJAVA0_IRM_ID				"*HWSJAV*"	/* Exit ID : HWSJAVA0				*/

/* MCI header area : Message Control Informations	*/

#define HWSJAVA0_HDR_MCI_LEN		0x0020		/* MCI Header :			32 bytes	*/

#define HWSJAVA0_OMCTLTNM_LEN		0x08		/* T-Pipe Name : 		 8 bytes	*/

/* SDF header area : State Data Format for Transaction-Related Information 	*/

#define HWSJAVA0_HDR_SDF_LEN		0x0048		/* SDF Header :			72 bytes	*/

#define HWSJAVA0_OMCTLMAP_LEN		0x08		/* Map Name : 			 8 bytes	*/
#define HWSJAVA0_OMHDRTOK_LEN		0x10		/* Server Token :		16 bytes	*/
#define HWSJAVA0_OMHDRCOR_LEN		0x10		/* Correlator Token :	16 bytes	*/
#define HWSJAVA0_OMHDRCID_LEN		0x10		/* Context ID :			16 bytes	*/
#define HWSJAVA0_OMHDRLTM_LEN		0x08		/* Dest. Override :		 8 bytes	*/

#define HWSJAVA0_MAP_NAME			"PIE01O"	/* Map Name							*/

/* SEC header area : Security Data 	*/

#define HWSJAVA0_HDR_SEC_LEN		0x006A		/* SEC Header :		   106 bytes	*/
#define HWSJAVA0_HDR_SEC_FILLER		0x0052		/* SEC Header Filler :	84 bytes	*/
												/* Arbtrary and temporary			*/

#define HWSJAVA0_OMSECRTY_LEN		0x01		/* User Token Type :	 1 byte		*/
#define HWSJAVA0_OMSECPRF_LEN		0x08		/* User Token :			 8 bytes	*/
#define HWSJAVA0_OMSECUTY_LEN		0x01		/* User ID Type :		 1 byte		*/
#define HWSJAVA0_OMSECUID_LEN		0x08		/* User ID :			 8 bytes	*/
#define HWSJAVA0_OMSECGTY_LEN		0x01		/* Groupe Name Type :	 1 byte		*/
#define HWSJAVA0_OMSECGRP_LEN		0x08		/* Groupe Name :		 8 bytes	*/

#define HWSJAVA0_SEC_FLAG_NONE		'N'			/* Flag : None						*/
#define HWSJAVA0_SEC_FLAG_CHECK		'C'			/* Flag : Check						*/
#define HWSJAVA0_SEC_FLAG_FULL		'F'			/* Flag : Full						*/
#define HWSJAVA0_SEC_TYPE_TOKEN		0x00		/* User Token Type					*/
#define HWSJAVA0_SEC_TYPE_USRID		0x02		/* User ID Type						*/
#define HWSJAVA0_SEC_TYPE_GROUP		0x03		/* Group Name Type					*/		

/* USR header area : User Data 		*/

#define HWSJAVA0_HDR_USR_LEN		0x0100		/* USR Header :		   256 bytes	*/
#define HWSJAVA0_HDR_USR_FILLER		0x00C0		/* USR Header filler : 192 bytes	*/

#define HWSJAVA0_HDR_USR_STR_LEN	0x08		/* USR Header strings :	 8 bytes	*/

#define HWSJAVA0_OMUSR_TRAN			0x00		/* Transaction Socket				*/
#define HWSJAVA0_OMUSR_PSOCKET		0x10		/* Persistent Socket				*/
#define HWSJAVA0_OMUSR_NPSOCKET		0x40		/* Non-persistent Socket			*/

/*********************************************************************
 * Data structures declarations
 */

/* MCI header area data : Message Control Informations	*/
 
typedef struct {
	
	char	archLevel;							/* Architecture Level		*/
	char	msgType;							/* Message Type				*/
	char	responseFlag;						/* Response Flag			*/
	char	commitFlag;							/* Commit Confirmation Flag	*/
	char	cmdType;							/* Command Type				*/
	char	processFlag;						/* Process Flag				*/
	char *	tPipeName;							/* Tpipe Name				*/
	char	chainFlag;							/* Chain Flag				*/
	char	prefixFlag;							/* Prefix Flag				*/
	int		sendSeqNb;							/* Send Sequence Number		*/
	short	senseCode;							/* Sense Code				*/
	short	reasonCode;							/* Reason Code				*/
	int		recoverSeqNb;						/* Recoverable Sequence Nbr	*/
	short	segmentSeqNb;						/* Segment Sequence Number	*/
	short	reserved;							/* Reserved					*/

} HWSJAVA0_headerMCI_t;

/* SDF header area data : State Data Format for Transaction-Related Information 	*/
 
typedef struct {
	
	short	length;								/* StateData Section Length	*/
	char	serverState;						/* Server State				*/
	char	syncFlag;							/* Synchronization Flag		*/
	char	syncLevel;							/* Synchronization Level	*/
	char	clientFlags;						/* Client Flags				*/
	char *	mapName;							/* Map Name					*/
	char *	serverToken;						/* Server Token				*/
	char *	correlToken;						/* Correlator Token			*/
	char *	contextId;							/* Context ID				*/
	char *	dstOverride;						/* Destination Override		*/
	short	srvUsrDataLen;						/* Server User Data Length	*/
	char *	srvUsrData;							/* Server User Data			*/

} HWSJAVA0_headerSDF_t;

/* SEC header area data : Security Data 	*/
 
typedef struct {
	
	short	length;								/* Security Section Length	*/
	char	securityFlag;						/* Security Flag			*/
	char	reserved;							/* Reserved					*/
	char	uTokenLen;							/* User Token Length		*/
	char	uTokenType;							/* User Token Type			*/
	char *	uToken;								/* User Token				*/
	char	userIdLen;							/* User ID Length			*/
	char	userIdType;							/* User ID Type				*/
	char *	userId;								/* User ID					*/
	char	groupeLen;							/* Groupe Length			*/
	char	groupeType;							/* Groupe Type				*/
	char *	groupe;								/* Groupe					*/
	char *	filler;								/* Header final filling		*/

} HWSJAVA0_headerSEC_t;

/* USR header area data : User Data 	*/
 
typedef struct {
	
	short	length;								/* User Data Section Length	*/
	short	fullWord;							/* Align FullWord			*/
	char *	destId;								/* Destination ID			*/
	char *	origId;								/* Origin ID				*/
	char *	portId;								/* Port ID					*/
	char *	lToken;								/* Logon Token				*/
	int		retCode;							/* Com. Return Code			*/
	char *	resCode;							/* Com. Reason Code			*/
	int		rToken;								/* Response Token			*/
	char *	passwd;								/* RACF Password			*/
	char	socket;								/* Socket Mode				*/
	char	pwdType;							/* Password Type			*/
	char	origSync;							/* Original Synchronization	*/
	char	timer;								/* Timer					*/
	/* Following attributes replaced by filler	*/
	char *	filler;								/* Header final filling		*/

} HWSJAVA0_headerUSR_t;

/* Exit module configuration	*/
 
typedef struct {
	
	HWSJAVA0_headerMCI_t	headerMCI;			/* MCI header area : Message Control Informations	*/
	HWSJAVA0_headerSDF_t	headerSDF;			/* SDF header area : State Data Format				*/
	HWSJAVA0_headerSEC_t	headerSEC;			/* SEC header area : Security Data					*/
	HWSJAVA0_headerUSR_t	headerUSR;			/* USR header area : User Data						*/

} HWSJAVA0_module_t;

/*********************************************************************
 * Functions prototypes declarations
 */

int		HWSJAVA0_setModule(IMS_factory_t *factory);
int		HWSJAVA0_getMsgLength(IMS_message_t *message);
int		HWSJAVA0_buildMessage(IMS_message_t *message, IMS_connect_t *params);
int		HWSJAVA0_writeMessage(IMS_message_t *message, binaryDataBuffer_t *dataBuffer);
int		HWSJAVA0_writeHeaders(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);
int		HWSJAVA0_writeHeaderMCI(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);
int		HWSJAVA0_writeHeaderSDF(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);
int		HWSJAVA0_writeHeaderSEC(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);
int		HWSJAVA0_writeHeaderUSR(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);
int		HWSJAVA0_readMessage(IMS_message_t *message, binaryDataBuffer_t *dataBuffer);

#endif
