/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: common.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <string.h>
/* Zend API library			*/
#include "php.h"					/* PHP : Zend API			*/
/* IMS extension libraries	*/
#include "common.h"					/* Common					*/


/**
 * String copy with right padding (blank filler)
 */
int _strPad(char *dest, int destLen, char *src, int srcLen, char filler)
{
	int	i;

	for (i = 0; i < srcLen && i < destLen; i++)
		dest[i] = src[i];
	while (i < destLen)
		dest[i++] = filler;
	return i;
}

/**
 * String generation
 * -> returns the new string or NULL if error
 */
char *_strBuild(char *src, int srcLen, int destLen, char filler)
{
	char *	dest;		/* Destination string	*/
	
	if (src == NULL) {	/* If source is NULL, completion with filler char	*/
		src		= "";
		srcLen	= 0;
	}
	if ((dest = (char *) emalloc(destLen)) == NULL)
		return NULL;
	if (_strPad(dest, destLen, src, srcLen, filler) < destLen) {
		efree(dest);
		return NULL;
	}
	return dest;
}

/**
 * String copy with encoding + right padding (blank filler)
 */
int _strEncode(char *dest, int destLen, char *src, int srcLen, char filler, char *encoder)
{
	int	i;

	if (encoder != NULL) {
		for (i = 0; i < srcLen && i < destLen; i++)
			dest[i] = encoder[(unsigned char) src[i]];
		while (i < destLen)
			dest[i++] = encoder[(unsigned char) filler];
	} else {
		for (i = 0; i < srcLen && i < destLen; i++)
			dest[i] = src[i];
		while (i < destLen)
			dest[i++] = filler;
	}
	return i;
}

/**
 * String equality test
 */
int _strIsEqual(char *str1, char *str2)
{
	return (strcmp(str1, str2) ? IMS_FALSE : IMS_TRUE);
}

/**
 * Error label recovery
 */
char *IMS_getErrorLabel(int errorCode)
{
	switch (errorCode)
	{
		case IMS_CODE_ERR_MALLOC :		return IMS_LABEL_ERR_MALLOC;
		case IMS_CODE_ERR_ENCODING :	return IMS_LABEL_ERR_ENCODING;
		case IMS_CODE_ERR_TRANSCODE :	return IMS_LABEL_ERR_TRANSCODE;
		case IMS_CODE_ERR_EXIT :		return IMS_LABEL_ERR_EXIT;
		case IMS_CODE_ERR_WRITING :		return IMS_LABEL_ERR_WRITING;
		case IMS_CODE_ERR_READING :		return IMS_LABEL_ERR_READING;
		case IMS_CODE_ERR_FIELD :		return IMS_LABEL_ERR_FIELD;
		case IMS_CODE_ERR_ATTRIBUTE :	return IMS_LABEL_ERR_ATTRIBUTE;
		case IMS_CODE_ERR_XML_FILE :	return IMS_LABEL_ERR_XML_FILE;
		case IMS_CODE_ERR_XML_INIT :	return IMS_LABEL_ERR_XML_INIT;
		case IMS_CODE_ERR_XML_PARSE :	return IMS_LABEL_ERR_XML_PARSE;
		case IMS_CODE_ERR_SEND :		return IMS_LABEL_ERR_SEND;
		case IMS_CODE_ERR_RECEIVE :		return IMS_LABEL_ERR_RECEIVE;
		case IMS_CODE_ERR_ANY :
		default :						return IMS_LABEL_ERR_ANY;
	}
}
