/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: connect.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
/* Zend API library			*/
#include "php.h"					/* PHP library				*/
/* PHP interface			*/
#include "php_ims_connect.h"		/* IMS Extension			*/
#include "php_ims_constants.h"		/* IMS Constants			*/
/* IMS extension libraries	*/
#include "common.h"					/* Common					*/
#include "connect.h"				/* IMS Connector			*/
#include "socket.h"					/* Socket streams			*/
#include "xml.h"					/* XML Processing			*/
#include "log.h"					/* Log & Debug				*/
/* Exit Modules				*/
#include "hwsimso0.h"				/* Simple module : HWSIMSO0	*/
#include "hwsjava0.h"				/* Java module :   HWSJAVA0	*/


/**
 * IMS connection parameters initialization
 */
void IMS_initConnect(IMS_connect_t *connect)
{
	connect->exitCode			= 0;
	connect->encoding			= NULL;
	connect->encodingLen		= 0;
	connect->msgRecord			= NULL;
	connect->msgRecordLen		= 0;
	connect->clientId			= NULL;
	connect->clientIdLen		= 0;
	connect->tranCode			= NULL;
	connect->tranCodeLen		= 0;
	connect->datastoreId		= NULL;
	connect->datastoreIdLen		= 0;
	connect->lterm				= NULL;
	connect->ltermLen			= 0;
	connect->racfUserId			= NULL;
	connect->racfUserIdLen		= 0;
	connect->racfGroupName		= NULL;
	connect->racfGroupNameLen	= 0;
	connect->racfPassword		= NULL;
	connect->racfPasswordLen	= 0;
}

/**
 * IMS message initialization
 */
void IMS_initMessage(IMS_message_t *message)
{
	message->encoder		= NULL;
	message->decoder		= NULL;
	message->msgLength		= 0;
	message->imsPrefix		= NULL;
	message->msgRecordLen	= 0;
	message->msgRecord		= NULL;
	message->modConfig		= NULL;
}

/**
 * IMS message manager initialization
 */
void IMS_initFactory(IMS_factory_t *factory)
{
	factory->connect	= NULL;
	factory->message	= NULL;
	factory->status		= IMS_MSG_EMPTY;	/* No message assigned		*/
	factory->build		= NULL;
	factory->write		= NULL;
	factory->read		= NULL;
}

/**
 * IMS message manager settings
 */
int IMS_setFactory(IMS_factory_t *factory, IMS_connect_t *connect, IMS_message_t *message)
{
	int	result	= IMS_CODE_OK;				/* Function return code		*/
	
	/* Connection parameters	*/
	factory->connect	= connect;
	/* IMS message data			*/
	factory->message	= message;
	/* Transcoding selection	*/
	if ((result = IMS_setEncoding(factory)) != IMS_CODE_OK)
	{
		return result;
	}
	/* Exit module selection	*/
	if ((result = IMS_setModule(factory)) != IMS_CODE_OK)
	{
		return result;
	}
	factory->status		= IMS_MSG_ALLOC;	/* No message assigned		*/
	
	return result;
}

/**
 * Exit module assignation to IMS message manager
 */
int IMS_setModule(IMS_factory_t *factory)
{
	switch (factory->connect->exitCode)
	{
		case IMS_CODE_HWSIMSO0 :	return HWSIMSO0_setModule(factory);
		case IMS_CODE_HWSJAVA0 :	return HWSJAVA0_setModule(factory);
		default :					return IMS_CODE_ERR_EXIT;
	}
}

/**
 * Transcoding assignation to IMS message manager
 */
int IMS_setEncoding(IMS_factory_t *factory)
{
	int	result;							/* Function return code		*/

/*	TODO : to remove / change
	switch (factory->connect->encoding)
	{
		case 0 :				return IMS_CODE_OK;
		case IMS_CODE_CP297 :	result = TRANSCODE_SET(cp297)(factory->encoder, factory->decoder);
								break;
		default :				return IMS_CODE_ERR_ENCODING;
	}
	if (result != TRANS_RESULT_OK) {
		return IMS_CODE_ERR_TRANSCODE;
	} else {
		factory->message->encoder = factory->encoder;
		factory->message->decoder = factory->decoder;
	}
*/
	TRANS_trans_t *	trans;				/* Transcoding to assign	*/
	
	if ((result = TRANS_getByName(&trans, factory->connect->encoding)) != TRANS_RESULT_OK)
	{
		return IMS_CODE_ERR_ENCODING;
	}
	else
	{
		factory->transcoder			= trans;
		factory->message->encoder	= trans->encoder;
		factory->message->decoder	= trans->decoder;
	}
	return IMS_CODE_OK;
}

/**
 * IMS message settings
 */
int IMS_buildMessage(IMS_factory_t *factory)
{
	int	result	= IMS_CODE_OK;			/* Function return code		*/
	
	/* IMS header prefix assignation	*/
	if ((result = IMS_initPrefix(factory->message)) != IMS_CODE_OK)
	{
		return result;
	}
	/* IMS Record data assignation		*/
	if (factory->connect->msgRecordLen && factory->connect->msgRecord)
	{
		factory->message->msgRecordLen	= factory->connect->msgRecordLen;
		factory->message->msgRecord		= factory->connect->msgRecord;
	}
	/* Data information					*/
	if ((result = factory->build(factory->message, factory->connect)) != IMS_CODE_OK)
	{
		return result;
	}
	/* Current message status update	*/
	factory->status = IMS_MSG_BUILT;

	return result;
}

/**
 * IMS message header prefix initialization
 */
int IMS_initPrefix(IMS_message_t *message)
{
	if ((message->imsPrefix = (IMS_prefix_t *) emalloc(sizeof(IMS_prefix_t))) == NULL)
	{
		return IMS_CODE_ERR_MALLOC;
	}
	message->imsPrefix->irmLen		= 0;
	message->imsPrefix->exitId		= NULL;
	message->imsPrefix->reserved	= 0;
	message->imsPrefix->timer		= 0;
	message->imsPrefix->socket		= 0;
	message->imsPrefix->encoding	= 0;
	message->imsPrefix->clientId	= NULL;
		
	return IMS_CODE_OK;
}	

/**
 * IMS message writing
 */
int IMS_writeMessage(IMS_factory_t *factory, binaryDataBuffer_t *dataBuffer)
{
	IMS_message_t *	message	= factory->message;	/* IMS message to write		*/
	int				result	= IMS_CODE_OK;		/* Function return code		*/
	int				written	= 0;				/* Written bytes size		*/

	/* Message data information if needed		*/
	if (factory->status != IMS_MSG_BUILT)
	{
		if ((result = IMS_buildMessage(factory)) != IMS_CODE_OK)
			return result;
	}
	/* BinaryDataBuffer initialization			*/
	if (binaryDataBuffer_initBuffer(dataBuffer, message->msgLength) == NULL)
	{
		return IMS_CODE_ERR_MALLOC;
	}
	/* Exit module specific message writing		*/
	written += factory->write(message, dataBuffer);
	/* Written size checking					*/
	if (message->msgLength == written && written == dataBuffer->dataOffset)
	{
		factory->status = IMS_MSG_WRITE;
		return IMS_CODE_OK;
	}
	else
	{
		factory->status = IMS_MSG_ERROR;
		return IMS_CODE_ERR_WRITING;
	}
}

/**
 * LLZZ tag writing
 */
int IMS_writeTagLLZZ(binaryDataBuffer_t *dataBuffer, IMS_message_t *message, int field_LL)
{
	int	written	= 0;					/* Written bytes size	*/

	/* LL field	*/
	written += binaryDataBuffer_writeShort(dataBuffer, field_LL);
	/* ZZ field	*/
	written += binaryDataBuffer_writeShort(dataBuffer, ZZ_TAG);

	return written;
}

/**
 * IMS message hedear prefix writing
 */
int IMS_writePrefix(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	int	written	= 0;					/* Written bytes size	*/

	IMS_prefix_t *	imsPrefix = message->imsPrefix;
	
	/* LLZZ tag				*/
	written += IMS_writeTagLLZZ(dataBuffer, message, imsPrefix->irmLen);
	/* IRM_ID field			*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, imsPrefix->exitId, HEADER_STR_LEN, message->encoder);
	/* RESERVED field		*/
	written += binaryDataBuffer_writeInt(dataBuffer, imsPrefix->reserved);
	/* IRM_F5 field			*/
	written += binaryDataBuffer_writeByte(dataBuffer, imsPrefix->msgType);
	/* IRM_TIMER field		*/
	written += binaryDataBuffer_writeByte(dataBuffer, imsPrefix->timer);
	/* IRM_SOCT field		*/
	written += binaryDataBuffer_writeByte(dataBuffer, imsPrefix->socket);
	/* IRM_ES	field		*/
	written += binaryDataBuffer_writeByte(dataBuffer, imsPrefix->encoding);
	/* IRM_CLIENTID field	*/
	written += binaryDataBuffer_encodeBytes(dataBuffer, imsPrefix->clientId, HEADER_STR_LEN, message->encoder);
	
	return written;
}

/**
 * IMS message data record writing
 */
int IMS_writeRecord(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	int	written	= 0;					/* Written bytes size	*/

	written += binaryDataBuffer_writeBytes(dataBuffer, message->msgRecord, message->msgRecordLen);
	
	return written;
}

/**
 * IMS message reading
 */
int IMS_readMessage(IMS_factory_t *factory, binaryDataBuffer_t *dataBuffer)
{
	IMS_message_t *	message	= factory->message;	/* IMS message to read	*/
	char			exitId[HEADER_STR_LEN];		/* Exit Module code		*/
	int				i, result = IMS_CODE_OK;	/* Function return code	*/

	/* Tests : Message status & Buffer length	*/
	if (factory->status != IMS_MSG_WRITE || dataBuffer->dataLength < message->imsPrefix->irmLen)
	{
		debug("IMS_readMessage() -> Tests failue");
		return IMS_CODE_ERR_READING;
	}
	/* Message size field reading				*/
	if (binaryDataBuffer_readInt(dataBuffer, &(message->msgLength)) != sizeof(int))
	{
		debug("IMS_readMessage() -> Message size reading failure");
		return IMS_CODE_ERR_READING;
	}
	/* Message Exit Module reading (decoding)	*/
	if (binaryDataBuffer_readBytes(dataBuffer, exitId, HEADER_STR_LEN) != HEADER_STR_LEN)
	{
		debug("IMS_readMessage() -> Exit Module code reading failure");
		return IMS_CODE_ERR_READING;
	}
	else if (TRANS_decode(exitId, HEADER_STR_LEN, factory->transcoder) != TRANS_RESULT_OK)
	{
		return IMS_CODE_ERR_READING;
	}
	/* Message Exit Module validation			*/
	for (i = 0; i < HEADER_STR_LEN; i++)
	{
		if (exitId[i] != message->imsPrefix->exitId[i])
		{
			debug("IMS_readMessage() -> Exit Module validation failure");
			return IMS_CODE_ERR_READING;
		}
	}
	/* Exit Module specific message reading		*/
	if ((result = factory->read(message, dataBuffer)) != IMS_CODE_OK)
	{
		debug("IMS_readMessage() -> Message reading failure");
		return result;	
	}
	/* Message record data reading				*/
	if ((result = IMS_readRecord(dataBuffer, message)) != IMS_CODE_OK)
	{
		debug("IMS_readMessage() -> Message data record reading failure");
		return result;
	}
	else
	{
		return IMS_CODE_OK;
	}
}

/**
 * IMS message record data reading
 */
int IMS_readRecord(binaryDataBuffer_t *dataBuffer, IMS_message_t *message)
{
	if ((message->msgRecord = (char *)emalloc(message->msgRecordLen)) == NULL)
	{
		return IMS_CODE_ERR_MALLOC;
	}
	else
	{
		binaryDataBuffer_readBytes(dataBuffer, message->msgRecord, message->msgRecordLen);
		return IMS_CODE_OK;
	}
}

/**
 * Character encoding
 */
char IMS_charEncode(char c, char *encoder)
{
	if (encoder != NULL)	/* Return with encoding		*/
	{
		return encoder[c];
	}
	else					/* Return without encoding	*/
	{
		return c;
	}
}

/**
 * Character decoding
 */
char IMS_charDecode(char c, char *decoder)
{
	if (decoder != NULL)	/* Return with decoding		*/
	{
		return decoder[c];
	}
	else					/* Return without decoding	*/
	{
		return c;
	}
}

/**
 * Data socket sending to IMS connect
 */
long IMS_send(php_stream *socketStream, binaryDataBuffer_t *dataBuffer)
{
	long	write_size	= 0;
	
	write_size = socket_send(socketStream, dataBuffer->dataBuffer, dataBuffer->dataLength);
/* TODO								*/
/* LOG display to remove			*/
char	buf[400];
snprintf(buf, sizeof(buf), "IMS_send() -> %d bytes", write_size);
debug(buf);
/************************************/
	return write_size;
}

/**
 * Data socket receiving from IMS Connect
 */
int IMS_receive(php_stream *socketStream, binaryDataBuffer_t *dataBuffer)
{
	char	buffer[IMS_BUFFER_SIZE];	/* Socket reading buffer		*/
	long	read_size	= 0;			/* Socket reading buffer size	*/
	long	read_length	= 0;			/* Reading total length			*/
	int		ims_length	= 0;			/* IMS data length				*/

	/* socket reading loop	*/
	do
	{
		/* Socket reading from IMS				*/
		read_size = socket_receive(socketStream, buffer, sizeof(buffer));
		/* First reading initializations		*/
		if (read_length == 0)
		{
			/* BinaryDataBuffer init			*/
			binaryDataBuffer_setBuffer(dataBuffer, buffer, read_size);
			/* IMS data total size				*/
			binaryDataBuffer_readInt(dataBuffer, &ims_length);
			/* Action depending on data size	*/
			if (read_size == ims_length)
			{
				/* End of reading if data complete	*/
				break;
			}
			else
			{
				/* Buffer init if data not complete	*/
				binaryDataBuffer_freeBuffer(dataBuffer);
				binaryDataBuffer_initBuffer(dataBuffer, ims_length);
			}
		}
		/* Reading data buffering	*/
		binaryDataBuffer_writeBytes(dataBuffer, buffer, read_size);
		read_length += read_size;
/* TODO								*/
/* LOG display to remove			*/
char	buf[400];
snprintf(buf, sizeof(buf), "IMS_receive() -> %d bytes (%d / %d)", read_size, read_length, ims_length);
debug(buf);
/************************************/
	}
	while (read_size && read_length < ims_length);
	
	/* BinaryDataBuffer rewind		*/
	binaryDataBuffer_setOffset(dataBuffer, 0x00);
	
	return read_length;
}

