/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: connect.h,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/*********************************************************************
 * Includes
 */

#include "binary_constants.h"				/* Binary writing macros	*/
#include "binaryDataBuffer.h"				/* Binary data buffering	*/
#include "transcoder.h"						/* IMS Transcoding			*/
#include "socket.h"							/* Socket streams			*/
#include "common.h"							/* Common					*/


#ifndef	_IMS_CONNECT_H_
#define	_IMS_CONNECT_H_

/*********************************************************************
 * Constants definitions
 */

/* Socket communication			*/

#define	IMS_BUFFER_SIZE		2048			/* IMS buffer size				*/

/* Header fields lengths		*/

#define	LLZZ_LEN			4				/* LLZZ tag length				*/
#define	TOTAL_LENGTH_LEN	4				/* Total size field length		*/
#define HEADER_STR_LEN		8				/* Header fields length			*/
#define	RECORD_HEAD_LEN		12				/* Record prefix length			*/

/* Header fields flag values	*/

#define	ZZ_TAG				0x0000			/* Common ZZ field value		*/
#define	LL_TAG_MSG_END		0x0004			/* Record ending LL field value	*/

#define	IRM_F5_OTMA_HEADERS	B8(10000000)	/* OTMA headers built by client	*/
#define	IRM_F5_CLIENT_TRANS	B8(01000000)	/* Translation done by client	*/
#define	IRM_F5_SINGLE_MSG	B8(00000001)	/* Single message				*/
#define	IRM_F5_AUTO_FLW		B8(00000010)	/* Auto Flow of messages		*/
#define	IRM_F5_NO_AUTO		B8(00000100)	/* No Auto Flow of messages		*/
#define	IRM_F5_NONE			B8(00000000)	/* None							*/

#define	MFS_MOD_OFF			B8(00000000)
#define	MFS_MOD_ON			B8(00100000)

#define	COMMIT_MODE_OFF		B8(01000000)
#define	COMMIT_MODE_ON		B8(00100000)

#define	SYNC_LEVEL_NONE		B8(00000000)
#define	SYNC_LEVEL_CONFIRM	B8(00000001)

#define	SYNC_FLAG_ACK		'A'
#define	SYNC_FLAG_NAK		'N'
#define	SYNC_FLAG_DEALLOC	'D'
#define	SYNC_FLAG_RESUME	'R'
#define	SYNC_FLAG_SENDONLY	'S'
#define	SYNC_FLAG_BLANK		B8(01000000)

/* Header fields string values	*/

#define IRM_ID_HWSIMSO0		"*IRMREQ*"		/* Exit Module code : SIMPLE 0	*/
#define IRM_ID_HWSIMSO1		"*IRMRE1*"		/* Exit Module code : SIMPLE 1	*/
#define IRM_ID_HWSJAVA0		"*HWSJAV*"		/* Exit Module code : JAVA		*/
#define IRM_ID_HWSCLSO1		"*IRMCSL*"
#define IRM_ID_HWSSMPL0		"*SAMPLE*"		/* Exit Module code : SAMPLE 0	*/
#define IRM_ID_HWSSMPL1		"*SAMPL1*"		/* Exit Module code : SAMPLE 1	*/

/* Transcoding direction codes	*/

#define IMS_ENCODE			1				/* Encoding : ASCII -> EBCDIC	*/
#define IMS_DECODE			2				/* Decoding : EBCDIC -> ASCII	*/

/* IMS message management codes	*/

#define IMS_MSG_EMPTY		'E'				/* Empty message			*/
#define IMS_MSG_ALLOC		'A'				/* Assigned message			*/
#define IMS_MSG_BUILT		'B'				/* Built message			*/
#define IMS_MSG_WRITE		'W'				/* Written message			*/
#define IMS_MSG_ERROR		'Z'				/* Error in message			*/


#define FILL_STR(dest, src)						IMS_strPad(dest, src, IMS_BLANK, HEADER_STR_LEN)
#define IMS_STRPAD(dest, destLen, src, srcLen)	IMS_strPadder(dest, destLen, src, srcLen, IMS_BLANK, HEADER_STR_LEN)

/*********************************************************************
 * Data structures declarations
 */

/*	IMS Connection parameters from PHP		*/
 
typedef struct {
	
	int		exitCode;						/* Exit Module code			*/
	char *	encoding;						/* Transcoding code			*/
	int		encodingLen;					/* Transcoding code length	*/
	char *	msgRecord;						/* Msg. record data			*/
	int		msgRecordLen;					/* Msg. record data length	*/
	char *	clientId;						/* IRM_CLIENTID value		*/
	int		clientIdLen;					/* CLIENTID value length	*/
	char *	tranCode;						/* IMS Transaction code		*/
	int		tranCodeLen;					/* IMS Transaction length	*/
	char *	datastoreId;					/* IRM_IMSDESTID value		*/
	int		datastoreIdLen;					/* IMSDESTID value length	*/
	char *	lterm;							/* IRM_LTERM value			*/
	int		ltermLen;						/* LTERM value length		*/
	char *	racfUserId;						/* IRM_RACF_USERID value	*/
	int		racfUserIdLen;					/* RACF_USERID value length	*/
	char *	racfGroupName;					/* IRM_RACF_GRNME value		*/
	int		racfGroupNameLen;				/* RACF_GRNME value length	*/
	char *	racfPassword;					/* IRM_RACF_PW value		*/
	int		racfPasswordLen;				/* RACF_PW value length		*/
/*	char *	racfAppName;					/* IRM_APPL_NM value		*/
/*	int		racfAppName;					/* APPL_NM value length		*/

} IMS_connect_t;

/* IMS message header prefix data			*/
 
typedef struct {
	
	int		irmLen;							/* IRM_LEN					*/
	char *	exitId;							/* IRM_ID					*/
	int		reserved;						/* Reserved use				*/
	char	msgType;						/* IRM_F5					*/
	char	timer;							/* IRM_TIMER				*/
	char	socket;							/* IRM_SOCT					*/
	char	encoding;						/* IRM_ES					*/
	char *	clientId;						/* IRM_CLIENTID				*/

} IMS_prefix_t;

/* IMS message data settings				*/
/* 	-> Message manager transcoding tables	*/
/* 	-> Exit Module configuration by code	*/
 
typedef struct {
	
	int				msgLength;				/* Message total length		*/
	IMS_prefix_t *	imsPrefix;				/* Message header prefix	*/
	int				msgRecordLen;			/* Record data length		*/
	char *			msgRecord;				/* Record data				*/

	char *			encoder;				/* IMS encoding table		*/
	char *			decoder;				/* IMS decoding table		*/
	void *			modConfig;				/* Exit module config		*/
											/*	-> set from Exit Code	*/
} IMS_message_t;

/* IMS message manager						*/
/* 	-> Transcoding tables set by code		*/
/* 	-> Transcoding function (not used) 		*/

typedef int (*IMS_encode_f)	(char *, int, int);							/* Type : IMS field encoder function pointer	*/
typedef int (*IMS_decode_f)	(char *, int, int);							/* Type : IMS field decoder function pointer	*/
typedef int (*IMS_build_f)	(IMS_message_t *, IMS_connect_t *);			/* Type : IMS message builder function pointer	*/
typedef int (*IMS_write_f)	(IMS_message_t *, binaryDataBuffer_t *);	/* Type : IMS message writer function pointer	*/
typedef int (*IMS_read_f)	(IMS_message_t *, binaryDataBuffer_t *);	/* Type : IMS message reader function pointer	*/

typedef struct {

	IMS_connect_t *	connect;				/* IMS Connection settings	*/
	IMS_message_t *	message;				/* IMS essage to process	*/
	char			status;					/* Current message status	*/
	
/*	char			encoder[TRAN_TBL_SIZE];	/* IMS Data encoding table	*/
/*	char			decoder[TRAN_TBL_SIZE];	/* IMS Data decoding table	*/
	TRANS_trans_t *	transcoder;				/* IMS Data transcoding		*/
	
/*	IMS_encode_f	encode;					/* IMS Field encode func.	*/
/*	IMS_decode_f	decode;					/* IMS Field decode func.	*/
	IMS_build_f		build;					/* IMS Message build func.	*/
	IMS_write_f		write;					/* IMS Message write func.	*/
	IMS_read_f		read;					/* IMS Message read func.	*/

} IMS_factory_t;

/*********************************************************************
 * Functions prototypes & macros declarations
 */

/* IMS messages management		*/

void	IMS_initConnect(IMS_connect_t *connect);
void	IMS_initFactory(IMS_factory_t *factory);
int		IMS_setFactory(IMS_factory_t *factory, IMS_connect_t *connect, IMS_message_t *message);
int		IMS_setModule(IMS_factory_t *factory);
int		IMS_setEncoding(IMS_factory_t *factory);
int		IMS_buildMessage(IMS_factory_t *factory);
int		IMS_writeMessage(IMS_factory_t *factory, binaryDataBuffer_t *dataBuffer);
int		IMS_readMessage(IMS_factory_t *factory, binaryDataBuffer_t *dataBuffer);

/* IMS messages processing		*/

void	IMS_initMessage(IMS_message_t *message);
int		IMS_initPrefix(IMS_message_t *message);
int		IMS_writeTagLLZZ(binaryDataBuffer_t *dataBuffer, IMS_message_t *message, int field_LL);
int		IMS_writePrefix(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);
int		IMS_writeRecord(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);
int		IMS_readRecord(binaryDataBuffer_t *dataBuffer, IMS_message_t *message);

/* IMS socket communication		*/

long	IMS_send(php_stream *socketStream, binaryDataBuffer_t *dataBuffer);
int		IMS_receive(php_stream *socketStream, binaryDataBuffer_t *dataBuffer);

/* IMS fields processing		*/

#define	IMS_makeField	_strBuild

char	IMS_charEncode(const char c, char *encoder);

/* Functions not used			*/

char	IMS_charDecode(const char c, char *decoder);

#endif
