/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: field.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* Zend API library			*/
#include "php.h"					/* PHP : Zend API			*/
/* IMS extension libraries	*/
#include "common.h"					/* Common					*/
#include "binaryDataBuffer.h"		/* Binary Data Buffer		*/
#include "transcoder.h"				/* IMS Transcoding			*/
#include "string.h"					/* IMS Strings				*/
#include "field.h"					/* IMS Data fields			*/
#include "log.h"					/* Log & Debug				*/


/**
 * New IMS data field
 */
IMS_field_t *IMS_newField()
{
	IMS_field_t *	field;			/* IMS data field			*/
	
	if (EMALLOC_IS_NULL(field, IMS_field_t)) {
		return NULL;
	} else {
		IMS_initField(field);
		return field;
	}
}

/**
 * IMS data field cloning
 * Incorrect & not used !
 */
IMS_field_t *IMS_cloneField(IMS_field_t *srcField)
{
	IMS_field_t *	newField;		/* New IMS data field		*/
	
	if (EMALLOC_IS_NULL(newField, IMS_field_t)) {
		return NULL;
	} else {
/*
		newField->name			= srcField->name;
		newField->length		= srcField->length;
		newField->segInit		= srcField->segInit;
		newField->attributes	= srcField->attributes;
		newField->filler		= srcField->filler;
		newField->align			= srcField->align;
		newField->type			= srcField->type;
		newField->prev			= srcField->prev;
		newField->next			= srcField->next;
*/		
		return newField;
	}
}

/**
 * IMS data field initialization
 */
int	IMS_initField(IMS_field_t *field)
{
	/* Field settings		*/
	
	field->name			= NULL;			/* Attribute : Field name	*/
	field->length		= 0;			/* Attribute : Length		*/
	field->segInit		= IMS_FALSE;	/* Attribute : Segment init	*/
	field->attribute	= IMS_FALSE;	/* Attribute : Attribute	*/
	field->prev			= NULL;			/* Previous field in list	*/
	field->next			= NULL;			/* Next field in list		*/

	/* Value properties		*/
	
	/* No type field		*/
	FIELD_TYPE(field)	= IMS_TYPE_NONE;
	/* CHAR type field		*/
	FIELD_VCHR(field)	= 0;
	/* SHORT type field		*/
	FIELD_VSHR(field)	= 0;
	/* INT type field		*/
	FIELD_VINT(field)	= 0;
	/* STRING type field	*/
	if (IMS_initString(&FIELD_VSTR(field), NULL, 0, IMS_BLANK, IMS_ALIGN_LEFT, NULL) != IMS_CODE_OK)
	{
		return IMS_CODE_ERR_FIELD;
	}
	return IMS_CODE_OK;
}

/**
 * IMS data field memory freeing
 */
void IMS_freeField(IMS_field_t *field)
{
	efree(field->name);
	if ((field->value).type == IMS_TYPE_STRING)
	{
		IMS_freeString(&(FIELD_VSTR(field)));
	}
}

/**
 * IMS data field insertion at the end of chained list
 */
IMS_field_t *IMS_insertField(IMS_field_t *newField, IMS_field_t **list_fields)
{
	IMS_field_t *	curField;		/* Current field in list	*/
	
	/* Last field in chained list	*/
	curField = *list_fields;
	while (curField && curField->next)
		curField = curField->next;
	/* New field insertion in list	*/
	newField->prev	= curField;
	newField->next	= NULL;
	if (curField)
		curField->next	= newField;
	else
		*list_fields	= newField;
	
	return newField;
}

/**
 * Recovery of next IMS data field in chained list
 */
IMS_field_t *IMS_nextField(IMS_field_t **f_list)
{
	/* Current field in list	*/
	IMS_field_t *	field = *f_list;
	/* Next field in list		*/
	if (field)
		*f_list = field->next;
		
	return field;
}

/**
 * IMS data field memory freeing from chained list
 */
void IMS_freeFieldsList(IMS_field_t **f_list)
{
	IMS_field_t **	list = f_list;	/* IMS fields chained list	*/
	IMS_field_t *	field;			/* Current field in list	*/
	
	while (field = IMS_nextField(list)) {
		IMS_freeField(field);
	}
	*f_list	= NULL;
}

/**
 * IMS data field type setting
 */
int IMS_setFieldType(IMS_field_t *field, char type)
{
	switch (type)
	{
		case IMS_TYPE_NONE :	FIELD_TYPE(field)	= IMS_TYPE_NONE;
								field->length		= 0;
								break;
		case IMS_TYPE_CHAR :	return IMS_setFieldChr(field, FIELD_VCHR(field));
		case IMS_TYPE_SHORT :	return IMS_setFieldShr(field, FIELD_VSHR(field));
		case IMS_TYPE_INT :		return IMS_setFieldInt(field, FIELD_VINT(field));
		case IMS_TYPE_STRING :	FIELD_TYPE(field)	= IMS_TYPE_STRING;
								break;
		default :				return IMS_CODE_ERR_FIELD;
	}
	return IMS_CODE_OK;
}

/**
 * IMS CHAR field value setting
 */
int IMS_setFieldChr(IMS_field_t *field, char value)
{
	FIELD_TYPE(field)	= IMS_TYPE_CHAR;
	FIELD_VCHR(field)	= value;
	field->length		= 1;
	return IMS_CODE_OK;
}

/**
 * IMS SHORT field value setting
 */
int IMS_setFieldShr(IMS_field_t *field, short value)
{
	FIELD_TYPE(field)	= IMS_TYPE_SHORT;
	FIELD_VSHR(field)	= value;
	field->length		= 2;
	return IMS_CODE_OK;
}

/**
 * IMS INT field value setting
 */
int IMS_setFieldInt(IMS_field_t *field, int value)
{
	FIELD_TYPE(field)	= IMS_TYPE_INT;
	FIELD_VINT(field)	= value;
	field->length		= 4;
	return IMS_CODE_OK;
}

/**
 * IMS STRING field value setting
 */
int IMS_setFieldStr(IMS_field_t *field, const char *src, int srcLen)
{
	int				result;			

	if ((result = IMS_setString(&FIELD_VSTR(field), src, srcLen)) != IMS_CODE_OK) {
		return result;
	} else {
		FIELD_TYPE(field)	= IMS_TYPE_STRING;
		return IMS_CODE_OK;
	}
}

/**
 * IMS data field value setting -> from CHAR type
 */
int	IMS_setField_char(IMS_field_t *field, char value)
{
	char buf[12];				/* String value buffer	*/
	switch (FIELD_TYPE(field))
	{
		case IMS_TYPE_CHAR :	return IMS_setFieldChr(field, value);
		case IMS_TYPE_SHORT :	return IMS_setFieldShr(field, value);
		case IMS_TYPE_INT :		return IMS_setFieldInt(field, value);
		case IMS_TYPE_STRING :	snprintf(buf, 12, "%c", value);
								return IMS_setFieldStr(field, buf, strlen(buf));
		case IMS_TYPE_NONE :
		default :				return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS data field value setting -> from SHORT type
 */
int	IMS_setField_short(IMS_field_t *field, short value)
{
	char buf[12];				/* String value buffer	*/
	switch (FIELD_TYPE(field))
	{
		case IMS_TYPE_CHAR :	return IMS_setFieldChr(field, (unsigned char)value);
		case IMS_TYPE_SHORT :	return IMS_setFieldShr(field, value);
		case IMS_TYPE_INT :		return IMS_setFieldInt(field, value);
		case IMS_TYPE_STRING :	snprintf(buf, 12, "%d", value);
								return IMS_setFieldStr(field, buf, strlen(buf));
		case IMS_TYPE_NONE :
		default :				return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS data field value setting -> from INT type
 */
int	IMS_setField_int(IMS_field_t *field, int value)
{
	char buf[12];				/* String value buffer	*/
	
	switch (FIELD_TYPE(field))
	{
		case IMS_TYPE_CHAR :	return IMS_setFieldChr(field, (unsigned char)value);
		case IMS_TYPE_SHORT :	return IMS_setFieldShr(field, (unsigned short)value);
		case IMS_TYPE_INT :		return IMS_setFieldInt(field, value);
		case IMS_TYPE_STRING :	snprintf(buf, 12, "%d", value);
								return IMS_setFieldStr(field, buf, strlen(buf));
		case IMS_TYPE_NONE :
		default :				return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS data field value setting -> from INT type
 */
int	IMS_setField_str(IMS_field_t *field, const char *src, int srcLen)
{
	switch (FIELD_TYPE(field))
	{
		case IMS_TYPE_CHAR :	return IMS_setFieldChr(field, src[0]);
		case IMS_TYPE_SHORT :	return IMS_setFieldShr(field, (unsigned short)atoi(src));
		case IMS_TYPE_INT :		return IMS_setFieldInt(field, atoi(src));
		case IMS_TYPE_STRING :	return IMS_setFieldStr(field, src, srcLen);
		case IMS_TYPE_NONE :
		default :				return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS data field value setting -> from VARIABLE type
 */
int IMS_setFieldVal(IMS_field_t *field, int type, const char *source)
{
	int	result;					/* Function return code	*/
	
	/* Starting segment field : LLZZ tag	*/
	if (field->segInit) {
		return IMS_setFieldInt(field, atoi(source));
	}
	/* IMS data field type setting			*/
	if ((result = IMS_setFieldType(field, type)) != IMS_CODE_OK) {
		return result;
	}
	/* IMS data field value setting			*/
	if (source) {
		return IMS_setField_str(field, source, strlen(source));
	}
}

/**
 * IMS CHAR field value recovery
 */
int	IMS_getFieldChar(IMS_field_t *field, char *vChr)
{
	if (FIELD_TYPE(field) == IMS_TYPE_CHAR) {
		*vChr = FIELD_VCHR(field);
		return IMS_CODE_OK;
	} else {
		*vChr = 0;
		return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS SHORT field value recovery
 */
int	IMS_getFieldShort(IMS_field_t *field, char *vShr)
{
	if (FIELD_TYPE(field) == IMS_TYPE_SHORT) {
		*vShr = FIELD_VSHR(field);
		return IMS_CODE_OK;
	} else {
		*vShr = 0;
		return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS INT field value recovery
 */
int	IMS_getFieldInt(IMS_field_t *field, char *vInt)
{
	if (FIELD_TYPE(field) == IMS_TYPE_INT) {
		*vInt = FIELD_VINT(field);
		return IMS_CODE_OK;
	} else {
		*vInt = 0;
		return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS STRING field value recovery
 */
int	IMS_getFieldStr(IMS_field_t *field, IMS_str_t *vStr)
{
	if (FIELD_TYPE(field) == IMS_TYPE_STRING) {
		*vStr = FIELD_VSTR(field);
		return IMS_CODE_OK;
	} else {
		IMS_initString(vStr, NULL, 0, IMS_BLANK, IMS_ALIGN_LEFT, NULL);
		return IMS_CODE_ERR_FIELD;
	}
}

/**
 * IMS data field writing
 */
int	IMS_writeField(IMS_field_t *field, binaryDataBuffer_t *dataBuffer)
{
	char *	buffer = NULL;			/* String writing buffer	*/
	int		result = IMS_CODE_OK;	/* Function return code		*/
	
	switch (FIELD_TYPE(field))
	{
		case IMS_TYPE_NONE :
			break;
		case IMS_TYPE_CHAR :
			if (field->length != binaryDataBuffer_writeByte(dataBuffer, FIELD_VCHR(field)))
				result = IMS_CODE_ERR_WRITING;
			break;
		case IMS_TYPE_SHORT :
			if (field->length != binaryDataBuffer_writeShort(dataBuffer, FIELD_VSHR(field)))
				result = IMS_CODE_ERR_WRITING;
			break;
		case IMS_TYPE_INT :		/* 16 bits shift for LLZZ tag fields	*/
			if (field->length != binaryDataBuffer_writeInt(dataBuffer, (field->segInit ? FIELD_INT_2_LLZZ(field) : FIELD_VINT(field))))
				result = IMS_CODE_ERR_WRITING;
			break;
		case IMS_TYPE_STRING :
			if (IMS_writeString(&FIELD_VSTR(field), &buffer, field->length) != IMS_CODE_OK)
				result = IMS_CODE_ERR_WRITING;
			if (buffer && field->length != binaryDataBuffer_writeBytes(dataBuffer, buffer, field->length))
				result = IMS_CODE_ERR_WRITING;
			if (buffer)
				efree(buffer);
			break;
		default :
			result = IMS_CODE_ERR_FIELD;
	}
	if (result != IMS_CODE_OK) {
		debug(IMS_getErrorLabel(result));
	}
	return result;
}

/**
 * IMS data field reading
 */
int	IMS_readField(IMS_field_t *field, binaryDataBuffer_t *dataBuffer)
{
	char *	buffer = NULL;			/* String reading buffer	*/
	int		result = IMS_CODE_OK;	/* Function return code		*/

	switch (FIELD_TYPE(field))
	{
		case IMS_TYPE_NONE :
			break;
		case IMS_TYPE_CHAR :
			if (field->length != binaryDataBuffer_readByte(dataBuffer, &FIELD_VCHR(field)))
				result = IMS_CODE_ERR_READING;
			break;
		case IMS_TYPE_SHORT :
			if (field->length != binaryDataBuffer_readShort(dataBuffer, &FIELD_VSHR(field)))
				result = IMS_CODE_ERR_READING;
			break;
		case IMS_TYPE_INT :		/* 16 bits shift for LLZZ tag fields	*/
			if (field->length != binaryDataBuffer_readInt(dataBuffer, &FIELD_VINT(field)))
				result = IMS_CODE_ERR_READING;
			if (field->segInit)
				FIELD_VINT(field) = FIELD_LLZZ_2_INT(field);
			break;
		case IMS_TYPE_STRING :
			if ((buffer = (char *) emalloc(field->length)) == NULL) {
				result = IMS_CODE_ERR_MALLOC;
			} else {
				result = binaryDataBuffer_readBytes(dataBuffer, buffer, field->length);
				result = IMS_readString(&FIELD_VSTR(field), buffer, result);
				efree(buffer);
			}
			break;
		default :
			result = IMS_CODE_ERR_FIELD;
	}
	if (result != IMS_CODE_OK) {
		debug(IMS_getErrorLabel(result));
	}
	return result;
}
