/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: field.h,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


#ifndef	_IMS_FIELD_H_
#define	_IMS_FIELD_H_

/*********************************************************************
 * Includes
 */

#include "string.h"					/* IMS Strings				*/
#include "binaryDataBuffer.h"		/* Binary Data Buffering	*/

/*********************************************************************
 * Constants definitions
 */

#define	IMS_TYPE_NONE		0		/* Undefined type			*/
#define	IMS_TYPE_CHAR		1		/* Type CHAR	-> 1 byte	*/
#define	IMS_TYPE_SHORT		2		/* Type SHORT	-> 2 bytes	*/
#define	IMS_TYPE_INT		3		/* Type INT		-> 4 bytes	*/
#define	IMS_TYPE_STRING		4		/* Type STRING	-> n bytes	*/

/*********************************************************************
 * Data structures declarations
 */

/* IMS data field value w/ type		*/

typedef struct {

	char			type;			/* Type de valeur IMS		*/
	char			vChr;			/* Type CHAR	-> 1 byte	*/
	short			vShr;			/* Type SHORT	-> 2 bytes	*/
	int				vInt;			/* Type INT		-> 4 bytes	*/
	IMS_str_t		vStr;			/* Type STRING	-> n bytes	*/

} IMS_value_t;

/* IMS data field (chained list)	*/

typedef struct IMS_field_s {

	char *				name;		/* Attribute : Field name	*/
	int					length;		/* Attribute : Length		*/
	char				segInit;	/* Attribute : Segment init	*/
	char				attribute;	/* Attribute : Attribute	*/
	IMS_value_t			value;		/* IMS data field value		*/

	struct IMS_field_s *prev;		/* Previous field in list	*/
	struct IMS_field_s *next;		/* Next field in list		*/
	
} IMS_field_t;

/* Access macros to fields values	*/

#define	FIELD_VAL_P(_field)		&(_field->value)
#define	FIELD_TYPE(_field)		((FIELD_VAL_P(_field))->type)
#define	FIELD_VCHR(_field)		((FIELD_VAL_P(_field))->vChr)
#define	FIELD_VSHR(_field)		((FIELD_VAL_P(_field))->vShr)
#define	FIELD_VINT(_field)		((FIELD_VAL_P(_field))->vInt)
#define	FIELD_VSTR(_field)		((FIELD_VAL_P(_field))->vStr)

#define	FIELD_INT_2_LLZZ(_field)	((FIELD_VINT(_field) & 0x0000ffff) << 16)
#define	FIELD_LLZZ_2_INT(_field)	((FIELD_VINT(_field) & 0xffff0000) >> 16)
/*
#define	FIELD_LLZZ(_field)		(int)( ((FIELD_VINT(_field) & 0xff00) << 24) | ((FIELD_VINT(_field) & 0x00ff) << 16) )
*/

/*********************************************************************
 * Functions prototypes declarations
 */

/* IMS data field processing	*/

IMS_field_t *	IMS_newField();
int				IMS_initField(IMS_field_t *field);
void			IMS_freeField(IMS_field_t *field);

/* IMS fields list processing 	*/

IMS_field_t *	IMS_insertField(IMS_field_t *newField, IMS_field_t **list_fields);
IMS_field_t *	IMS_nextField(IMS_field_t **list_fields);
void			IMS_freeFieldsList(IMS_field_t **f_list);

/* IMS fields values processing	*/

int	IMS_setFieldChr(IMS_field_t *field, char value);
int	IMS_setFieldShr(IMS_field_t *field, short value);
int	IMS_setFieldInt(IMS_field_t *field, int value);
int	IMS_setFieldStr(IMS_field_t *field, const char *src, int srcLen);
int	IMS_setField_char(IMS_field_t *field, char value);
int	IMS_setField_short(IMS_field_t *field, short value);
int	IMS_setField_int(IMS_field_t *field, int value);
int	IMS_setField_str(IMS_field_t *field, const char *src, int srcLen);
int	IMS_setFieldVal(IMS_field_t *field, int type, const char *source);

int	IMS_getFieldChar(IMS_field_t *field, char *vChr);
int	IMS_getFieldShort(IMS_field_t *field, char *vShr);
int	IMS_getFieldInt(IMS_field_t *field, char *vInt);
int	IMS_getFieldStr(IMS_field_t *field, IMS_str_t *vStr);

/* IMS fields writing and reading	*/

int	IMS_readField(IMS_field_t *field, binaryDataBuffer_t *dataBuffer);
int	IMS_writeField(IMS_field_t *field, binaryDataBuffer_t *dataBuffer);

/* Not used functions				*/

IMS_field_t *	IMS_cloneField(IMS_field_t *srcField);

#endif
