/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: socket.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
/* Zend API library			*/
#include "php.h"					/* PHP : Zend API			*/
#include "php_network.h"			/* PHP : Network			*/

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#include <sys/types.h>
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef PHP_WIN32
#include <winsock.h>
#elif defined(NETWARE)
#ifdef NEW_LIBC
#ifdef USE_WINSOCK
#include <novsock2.h>
#else
#include <netinet/in.h>
#include <netdb.h>
#include <sys/select.h>
#endif
#endif
#else
#include <netinet/in.h>
#include <netdb.h>
#if HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif
#endif
#if defined(PHP_WIN32) || defined(__riscos__) || defined(NETWARE)
#undef AF_UNIX
#endif
#if defined(AF_UNIX)
#include <sys/un.h>
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#ifndef PF_INET
#define PF_INET AF_INET
#endif

#ifndef PF_UNIX
#define PF_UNIX AF_UNIX
#endif

#ifdef PHP_WIN32
#define EWOULDBLOCK WSAEWOULDBLOCK
#elif defined(NETWARE)
#ifdef USE_WINSOCK
#define EWOULDBLOCK WSAEWOULDBLOCK
#endif
#endif

/* IMS extension libraries	*/
#include "socket.h"					/* Socket streams			*/


/**
 * Connection opening -> returns socket stream
 */
php_stream *socket_connect(socket_connection_info *connection)
{
	php_stream	*stream = NULL;		/* PHP socket stream		*/
	
	/* Socket stream opening		*/
	stream = php_stream_sock_open_host(connection->host, (unsigned short)connection->port, connection->socktype, connection->timeout, connection->persistent_id);
	/* Stream conversion			*/
/* TODO								*/
/* HTML display to remove
/*
	if (php_stream_is(socket, PHP_STREAM_IS_STDIO)) {
debug("socket_connect() -> Socket is STDIO");
	}
	if (php_stream_is(socket, PHP_STREAM_IS_SOCKET)) {
debug("socket_connect() -> Socket is SOCKETD");
	}
debug("socket_connect() -> Socket TEST cast STDIO");
	if (php_stream_can_cast(socket, PHP_STREAM_AS_STDIO)) {
debug("socket_connect() -> Socket can cast STDIO");
	}
debug("socket_connect() -> Socket TEST cast SOCKETD");
	if (php_stream_can_cast(socket, PHP_STREAM_AS_SOCKETD)) {
debug("socket_connect() -> Socket can cast SOCKETD");
	}
debug("socket_connect() -> Socket CAST");
	if (php_stream_cast(socket, PHP_STREAM_AS_STDIO, (void*)&stream, REPORT_ERRORS) == FAILURE) {
		return NULL;
	}
*/
/* TODO								*/
/* HTML display to remove			*/
/*	
	if (stream)
		php_printf("socket_connect : OK <br>");
	else
		php_printf("socket_connect : KO <br>");
/************************************/

	return stream;
}

/**
 * Socket connection closure
 */
int socket_disconnect(php_stream *stream)
{
	if (stream)
	{
		return php_stream_close(stream);
	}
	else
	{
		return 0;
	}
}

/**
 * Socket stream data sending -> returns sent data size
 */
long socket_send(php_stream *stream, const char *data, int data_len)
{
	size_t	count	= (size_t) data_len;
	size_t	out		= 0;
	
	if (!stream)
	{
		return 0;
	}
	out = php_stream_write(stream, data, count);

/* TODO								*/
/* HTML display to remove			*/
/*	
	php_printf("<BR> socket_send() -> %d bytes envoys <BR>", out);
/************************************/
	
	return out;
}

/**
 * Socket stream data receiving -> returns received data size
 * Caution : Failure of EOF test -> response altency
 */
long socket_receive(php_stream *stream, char *data, int data_len)
{
debug("socket_receive() -> START");
	size_t	count	= (size_t) data_len;
	size_t	in		= 0;
	
	if (!stream)
	{
		return 0;
	}
/* TODO								*/
/* Caution : EOF test failure !		*/
	if (php_stream_eof(stream) == 0)
	{
		in = php_stream_read(stream, data, count);
	}
/* TODO								*/
/* LOG display to remove			*/
char	buf[400];
snprintf(buf, sizeof(buf), "socket_receive() -> %d bytes", in);
debug(buf);
/************************************/

debug("socket_receive() -> END");
	return in;
}

/**
 * Data reception into binary Data Buffer
 */
long socket_receive_to_buffer(php_stream *stream, binaryDataBuffer_t *dataBuffer)
{
debug("socket_receive_to_buffer() -> START");
	size_t	length;
	char *	tmpBuffer, result;
	
	if (length = php_stream_copy_to_mem(stream, &tmpBuffer, PHP_STREAM_COPY_ALL, 0))
	{
debug("socket_receive_to_buffer() -> Bufferized");
		binaryDataBuffer_setBuffer(dataBuffer, tmpBuffer, length);
	}
	if (tmpBuffer)
	{
		efree(tmpBuffer);
	}
debug("socket_receive_to_buffer() -> END");
	return dataBuffer->dataBuffer ? length : 0;
}
