/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: transcoder.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* IMS extension libraries	*/
#include "common.h"					/* Common					*/
#include "transcoder.h"				/* IMS Transcoding			*/
#include "log.h"					/* Log & Debug				*/


/*********************************************************************
 * Transcoding context processing
 *	-> Global variable declaration /!\
 */

/* Global transcoding context declaration	*/

TRANS_ref_t	TRANS_REF_GBL;

/**
 * Global transcoding context recovery
 */
TRANS_ref_t * TRANS_get_ref()
{
	return &TRANS_REF_GBL;
}

/**
 * Global transcoding context reset
 */
int TRANS_reset()
{
	TRANS_REF_GBL.counter	= 0;
	TRANS_REF_GBL.builders	= NULL;
	TRANS_REF_GBL.trans_arr	= NULL;
	TRANS_REF_GBL.trans_def	= NULL;
}

/**
 * Global transcoding context initialization
 */
int TRANS_init()
{
	TRANS_trans_t *	array;					/* Translaters array		*/
	TRANS_list_t *	builder;				/* Translater builders list	*/
	TRANS_build_t	directBuild;			/* For direct translater	*/
	int				size;					/* Array size				*/

TRANS_log(TRANS_RESULT_OK, "TRANS_init() -> START", NULL);

	/* Transcoding context initialization	*/
	
	TRANS_reset();							/* Context reset			*/

TRANS_log(TRANS_RESULT_OK, "TRANS_init() -> TRANSCODERS INIT", NULL);

	SET_TRANSCODERS;						/* Translaters declaration	*/
	builder	= TRANS_REF_GBL.builders;
	size	= TRANS_REF_GBL.counter + 1;	/* For direct translater	*/
	
	/* Array memory allocation				*/
	
TRANS_log(TRANS_RESULT_OK, "TRANS_init() -> TRANSCODERS ARRAY", NULL);

	if ((array = (TRANS_trans_t *) malloc(size * sizeof(TRANS_trans_t))) == NULL) {
		return TRANS_log(TRANS_ERR_MALLOC, "TRANS_init() -> Malloc failure", NULL);
	} else {
		TRANS_REF_GBL.trans_arr	= array;
	}
	TRANS_REF_GBL.counter = 0;				/* Couter reset				*/
	
	/* Direct translater creation			*/
	
TRANS_log(TRANS_RESULT_OK, "TRANS_init() -> DIRECT TRANSCODER", NULL);

	directBuild.name		= TRANS_DIRECT_NAME;		/* Default name				*/
	directBuild.encoder		= NULL;						/* No encoding				*/
	directBuild.enc_size	= TRANS_TBL_SIZE;			/* Tests validation			*/
	directBuild.decoder		= NULL;						/* No decoding				*/
	directBuild.dec_size	= TRANS_TBL_SIZE;			/* Tests validation			*/
	directBuild.isDefault	= TRANS_TRUE;				/* Default translater		*/
	directBuild.isDirect	= TRANS_TRUE;				/* No conversion processing	*/
	
	TRANS_build(&directBuild);				/* Direct translater creation	*/
	
	/* Translaters creation					*/
	
TRANS_log(TRANS_RESULT_OK, "TRANS_init() -> TRANSCODERS BUILDING", NULL);

	while (builder != NULL) {
		builder->build();								/* Translater creation		*/
		builder	= builder->next;						/* Shift to next translater	*/
	}

	/* Transcoding context update			*/
	
TRANS_log(TRANS_RESULT_OK, "TRANS_init() -> UPDATE REFERENTIEL", NULL);

	TRANS_delete_builders(&(TRANS_REF_GBL.builders));	/* Translaters deletion		*/
	
TRANS_log(TRANS_RESULT_OK, "TRANS_init() -> END", NULL);

	return TRANS_RESULT_OK;
}

/**
 * Transcoding context memory freeing
 */
int TRANS_free()
{
	int	index;		/* Translater index		*/
	
	/* Translaters name memory freeing		*/
	for (index = 0; index < TRANS_REF_GBL.counter; index++) {
		free(TRANS_REF_GBL.trans_arr[index].name);
	}
	/* Translaters array memory freeing		*/
	free(TRANS_REF_GBL.trans_arr);
}

/**
 * New translater declaration
 */
int TRANS_declare(TRANS_builder builder)
{
	TRANS_list_t **	builders = &(TRANS_REF_GBL.builders);
	
	TRANS_REF_GBL.counter++;
	
	return TRANS_insert_builder(builders, builder);
}

/**
 * New translater builder insertion into chained list
 */
int TRANS_insert_builder(TRANS_list_t **builders, TRANS_builder builder)
{
	TRANS_list_t *	newItem;			/* Item to insert			*/

TRANS_log(TRANS_RESULT_OK, "TRANS_insert_builder() -> builder", NULL);
	if ((newItem = (TRANS_list_t *) malloc(sizeof(TRANS_list_t))) == NULL) {
		return TRANS_log(TRANS_ERR_MALLOC, "TRANS_insert_builder() -> Malloc failure", NULL);
	}	
	newItem->build	= builder;			/* Builder assignation		*/
	newItem->next	= *builders;		/* List assignation			*/
	*builders		= newItem;			/* Builder insertion		*/

	return TRANS_RESULT_OK;
}

/**
 * Translater builder deletion from chained list
 */
int TRANS_delete_builders(TRANS_list_t **builders)
{
	TRANS_list_t *	item;				/* Item to delete			*/
	
	char	buf[50];
	
	while (*builders != NULL) {
		item		= *builders;		/* List assignation			*/
snprintf(buf, 50, "%d", (int)item);
TRANS_log(TRANS_RESULT_OK, "TRANS_delete_builders() -> item = %s", buf);
		*builders	= item->next;		/* Iem deletion				*/
snprintf(buf, 50, "%d", (int)(*builders));
TRANS_log(TRANS_RESULT_OK, "TRANS_delete_builders() -> next = %s", buf);
snprintf(buf, 50, "%d", (int)item);
TRANS_log(TRANS_RESULT_OK, "TRANS_delete_builders() -> item = %s", buf);
		free(item);						/* Item memory freeing		*/
	}
	return TRANS_RESULT_OK;
}

/**
 * Translater creation & insertion into transcoding context
 */
int TRANS_build(TRANS_build_t *building)
{
	TRANS_trans_t *	trans;				/* New translater			*/
	char *			name;				/* Translater name			*/
	int				i;
	
	TRANS_log(TRANS_RESULT_OK, "TRANS_build() -> START [%s]", building->name);

	/* Conversion tables size validation	*/
	if (building->enc_size != TRANS_TBL_SIZE || building->dec_size != TRANS_TBL_SIZE) {
		return TRANS_log(TRANS_ERR_SRC_TBL, "TRANS_build() -> Table size error [%s]", building->name);
	}
	/* Translater name memory allocation	*/
	if ((name = (char *) malloc(strlen(building->name) + 1)) == NULL) {
		free(trans);
		return TRANS_log(TRANS_ERR_MALLOC, "TRANS_build() -> Name malloc error [%s]", building->name);
	}
	/* Translater settings -> index, name and tables	*/
	trans			= &((TRANS_REF_GBL.trans_arr)[TRANS_REF_GBL.counter]);
	trans->index	= TRANS_REF_GBL.counter;
	trans->name		= strncpy(name, building->name, strlen(building->name) + 1);

	if (building->isDirect) {
		/* No charconversion for direct translater		*/
		for (i = 0; i < TRANS_TBL_SIZE; i++) {
			(trans->encoder)[i]	= (char) i;
			(trans->decoder)[i]	= (char) i;
		}
	} else {
		/* Conversion talbes copy	*/
		memcpy(trans->encoder, building->encoder, TRANS_TBL_SIZE);
		memcpy(trans->decoder, building->decoder, TRANS_TBL_SIZE);
	}
	/* Transcoding context update	*/
	if (building->isDirect || building->isDefault) {		/* If direct or default translater	*/
		TRANS_REF_GBL.trans_def = trans;					/* Direct translater definition		*/
	}
	TRANS_REF_GBL.counter++;								/* Translaters counter update		*/
	
	TRANS_log(TRANS_RESULT_OK, "TRANS_build() -> END [%s]", trans->name);

	return TRANS_RESULT_OK;
}

/**
 * Error tracing in APACHE log
 * -> Doesn't work !!!
 */
int TRANS_phpLog(int error, const char *format, const char *detail)
{
	char msg[TRANS_ERR_SIZE];			/* Error message buffer		*/

	snprintf((msg + snprintf(msg, TRANS_ERR_SIZE, format, detail)), TRANS_ERR_SIZE, " {ERROR=%d}", error);
/*
	php_log_err(msg TSRMLS_CC);
*/
	return error;
}

/**
 * Error tracing in applicative debug log
 */
int TRANS_log(int error, const char *format, const char *detail)
{
	char msg[TRANS_ERR_SIZE];			/* Error message buffer		*/

	snprintf((msg + snprintf(msg, TRANS_ERR_SIZE, format, detail)), TRANS_ERR_SIZE, " {ERROR=%d}", error);
	debug(msg);
	return error;
}

/*********************************************************************
 * Translater selection / Transcoding processing
 */

/**
 * Translater search from index
 */
int	TRANS_getByIndex(TRANS_trans_t **trans, int index)
{
	char strIndex[13];					/* Index writing buffer		*/
	
	if (index < TRANS_REF_GBL.counter && index >= 0)
	{
		*trans = &(TRANS_REF_GBL.trans_arr[index]);	/* Target translater	*/
		return TRANS_RESULT_OK;
	} else {
		*trans = TRANS_REF_GBL.trans_def;			/* Default translater	*/
		sprintf(strIndex, "%d", index);
		return TRANS_log(TRANS_ERR_UNKNOWN, "TRANS_getByIndex() -> Unknown translater error [%s]", strIndex);
	}
}

/**
 * Translater search from name
 */
int TRANS_getByName(TRANS_trans_t **trans, const char *name)
{
	int	index;							/* Translater index		*/
	
	/* Default translater tests : name = DEFAULT ou missing		*/
	
	if (!name || !strlen(name) || !strcmp(name, TRANS_DEFAULT_NAME)) {
		*trans = TRANS_REF_GBL.trans_def;
		return TRANS_RESULT_OK;
	}
	/* Translaters names tests in transcoding context	*/
	
	for (index = 0; index < TRANS_REF_GBL.counter; index++)
	{
		if (STR_IS_EQUAL(name, TRANS_REF_GBL.trans_arr[index].name))
		{
			/* Search successful	*/
			*trans = &(TRANS_REF_GBL.trans_arr[index]);
			return TRANS_RESULT_OK;
		}
	}
	/* If search failure : default translater selection	*/
	
	*trans = TRANS_REF_GBL.trans_def;

	return TRANS_log(TRANS_ERR_UNKNOWN, "TRANS_getByName() -> Unknown translater error [%s]", name);
}

/**
 * Translater selection from name
 */
TRANS_trans_t *	TRANS_select(const char *name)
{
	TRANS_trans_t *	trans;			 /* Translater to select	*/
	
	TRANS_getByName(&trans, name);
	return trans;
}

/**
 * String encoding
 */
int TRANS_encode(char *str, int len, TRANS_trans_t *trans)
{
	return TRANS_transcode(str, len, trans, TRANS_ENCODING);
}

/**
 * String decoding
 */
int TRANS_decode(char *str, int len, TRANS_trans_t *trans)
{
	return TRANS_transcode(str, len, trans, TRANS_DECODING);
}

/**
 * String transcoding -> both ways (encoding / decoding)
 */
int TRANS_transcode(char *str, int len, TRANS_trans_t *trans, char type)
{
	char *	coder;		/* Transcoding table	*/
	int		index;		/* String offset		*/
	
	/* Transcoding table selection				*/

	switch (type)
	{
		case TRANS_ENCODING :	coder = trans->encoder;
								break;
		case TRANS_DECODING :	coder = trans->decoder;
								break;
		default :
			return TRANS_log(TRANS_ERR_TYPE, "TRANS_transcode() -> Invalid transcoding error [%s]", trans->name);
	}
	if (coder == NULL) {
		return TRANS_log(TRANS_ERR_SRC_TBL, "TRANS_transcode() -> Missing table error [%s]", trans->name);
	}
	
	/* String transcoding						*/
	
	for (index = 0; index < len; index++) {
		str[index] = coder[(unsigned char) str[index]];
	}
	return TRANS_RESULT_OK;
}
