/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: xml.h,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/*********************************************************************
 * Includes
 */

#include <expat.h>							/* EXPAT parsing library	*/


#ifndef	_IMS_XML_H_
#define	_IMS_XML_H_

/*********************************************************************
 * Constants definitions
 */

#define XML_BUFFER_SIZE			2048

/* Error codes	*/

#define	XML_CODE_OK				0
#define	XML_CODE_ERR_INIT		-10
#define	XML_CODE_ERR_FILE		-20
#define	XML_CODE_ERR_PARSE		-30

/*********************************************************************
 * Data structures declarations
 */

/* XML element attribute	*/

typedef struct {

	char *		name;						/* Attribute name			*/
	char *		value;						/* Attribute value			*/

} IMS_xmlAttr_t;

/* XML parsing data			*/

typedef struct {

	char *			filename;				/* File name to parse		*/
	int				depth;					/* XML parsing depth		*/
	char *			tag;					/* Current XML element tag	*/
	char **			atts;					/* Current XML attributes	*/
	void *			wrapper;				/* XML message Wrapper		*/
	void *			config;					/* XML parsing config		*/

} IMS_xmlData_t;

/* XML element tag handler	*/

typedef	int	(*IMS_xml_handler)(IMS_xmlData_t *xml);

/* XML parsing config		*/

typedef struct {
	
	char *			tag;					/* XML element tag name		*/
	IMS_xml_handler	start;					/* Opening tag handler		*/
	IMS_xml_handler	end;					/* Closing tag handler		*/
	
} IMS_xmlConf_t;

/*********************************************************************
 * Functions prototypes & macros declarations
 */

#define	IMS_CAST_XML(_dest, _src)	IMS_xmlData_t *	_dest = (IMS_xmlData_t *)(_src)

IMS_xmlConf_t	IMS_xml_config(char *tag, IMS_xml_handler start, IMS_xml_handler end);
int				IMS_xml_initData(IMS_xmlData_t *xml_data, const char *filename, void *wrapper, IMS_xmlConf_t *config);
int				IMS_xml_initParser(IMS_xmlData_t *xml_data, XML_Parser *xmlParser);
int				IMS_xml_parse(IMS_xmlData_t *xml_data);
void			IMS_xml_startElement(void *userData, const char *name, const char **atts);
int				IMS_xml_nextAttribute(IMS_xmlAttr_t *ps_attribute, char ***pa_attributes);
void			IMS_xml_endElement(void *userData, const char *name);
void			IMS_xml_charData(void *userData, const XML_Char *data, int length);
void			IMS_xml_startCData(void *userData);
void			IMS_xml_endCData(void *userData);

#endif
