/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: php_ims_connect.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* Standard libraries		*/
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
/* Zend API library			*/
#include "php.h"					/* PHP : Zend API			*/
#include "SAPI.h"					/* PHP : SAPI				*/
#include "php_network.h"			/* PHP : Network			*/
#include "php_globals.h"			/* PHP : Globals			*/

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#include <sys/types.h>
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef PHP_WIN32
#include <winsock.h>
#elif defined(NETWARE)
#ifdef NEW_LIBC
#ifdef USE_WINSOCK
#include <novsock2.h>
#else
#include <netinet/in.h>
#include <netdb.h>
#include <sys/select.h>
#endif
#endif
#else
#include <netinet/in.h>
#include <netdb.h>
#if HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif
#endif
#if defined(PHP_WIN32) || defined(__riscos__) || defined(NETWARE)
#undef AF_UNIX
#endif
#if defined(AF_UNIX)
#include <sys/un.h>
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#ifndef PF_INET
#define PF_INET AF_INET
#endif

#ifndef PF_UNIX
#define PF_UNIX AF_UNIX
#endif

#ifdef PHP_WIN32
#define EWOULDBLOCK WSAEWOULDBLOCK
#elif defined(NETWARE)
#ifdef USE_WINSOCK
#define EWOULDBLOCK WSAEWOULDBLOCK
#endif
#endif

/* PHP interface			*/
#include "php_ims_connect.h"		/* IMS Extension			*/
#include "php_ims_constants.h"		/* IMS Constants			*/
/* IMS extension libraries	*/
#include "connect.h"				/* IMS Connector			*/
#include "socket.h"					/* Socket streams			*/
#include "field.h"					/* IMS Data fields			*/
#include "msg_wrap.h"				/* Messages XML formating	*/
#include "log.h"					/* Log & Debug				*/


ZEND_DECLARE_MODULE_GLOBALS(ims_connect)

static function_entry ims_connect_functions[] = {
	PHP_FE(ims_connect, NULL)
	PHP_FE(ims_wrap_message, NULL)
	PHP_FE(ims_unwrap_message, NULL)
	PHP_FE(ims_execute, NULL)
	PHP_FE(ims_encode, NULL)
	PHP_FE(ims_decode, NULL)
	PHP_FE(ims_exec_log, NULL)
	PHP_FE(ims_print_hexa, NULL)
	{NULL, NULL, NULL}
};

zend_module_entry ims_connect_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	PHP_IMS_CONNECT_EXTNAME,
	ims_connect_functions,
	PHP_MINIT(ims_connect),
	PHP_MSHUTDOWN(ims_connect),
	PHP_RINIT(ims_connect),
	PHP_RSHUTDOWN(ims_connect),
	PHP_MINFO(ims_connect),
#if ZEND_MODULE_API_NO >= 20010901
	PHP_IMS_CONNECT_VERSION,
#endif
	STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_IMS_CONNECT
ZEND_GET_MODULE(ims_connect)
#endif

/**
 * Request global variables initializations
 */
static void php_ims_connect_init_globals(zend_ims_connect_globals *ims_connect_globals)
{
}

/**
 * Extension module initialization
 */
PHP_MINIT_FUNCTION(ims_connect)
{
	debug("PHP_MINIT_FUNCTION -> START");
	
	/* Request global variables initialization	*/
	ZEND_INIT_MODULE_GLOBALS(ims_connect, php_ims_connect_init_globals, NULL);
	
	/* Transcodings initialization				*/
	TRANS_init();
	debug("PHP_MINIT_FUNCTION -> TRANS INIT DONE");
	
	/* Exit codes declarations					*/
	REGISTER_LONG_CONSTANT(PHP_CONSTANT_HWSIMSO0, IMS_CODE_HWSIMSO0, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT(PHP_CONSTANT_HWSJAVA0, IMS_CODE_HWSJAVA0, CONST_CS | CONST_PERSISTENT);

	/* Encoding types declarations				*/
	REGISTER_LONG_CONSTANT(PHP_CONSTANT_CP297, IMS_CODE_CP297, CONST_CS | CONST_PERSISTENT);

	debug("PHP_MINIT_FUNCTION -> END");
	return SUCCESS;
}

/**
 * Extension module shutdown
 */
PHP_MSHUTDOWN_FUNCTION(ims_connect)
{
	/* Transcodings memeory freeing	*/
	TRANS_free();
	
	return SUCCESS;
}

/**
 * Request initialization
 */
PHP_RINIT_FUNCTION(ims_connect)
{
	debug("PHP_RINIT_FUNCTION -> START");

	/* IMS execution log initialization	*/
	
	MAKE_STD_ZVAL(IMS_CONNECT_G(IMS_execLog));
	if(array_init(IMS_CONNECT_G(IMS_execLog)) != SUCCESS)
	{
		/* Error managing	*/
		debug("PHP_RINIT -> Execution log failed to init");
		IMS_CONNECT_G(IMS_execLog) = NULL;
		
		return FAILURE;
	}
	return SUCCESS;
}

/**
 * Request shutdown
 */
PHP_RSHUTDOWN_FUNCTION(ims_connect)
{
	return SUCCESS;
}

/**
 * Module informations
 */
PHP_MINFO_FUNCTION(ims_connect)
{
	char buf[200];

	php_info_print_table_start();
	php_info_print_table_header(2, "IMS Support", "enabled");
	php_info_print_table_row(2, "Build Version", PHP_IMS_CONNECT_VERSION);
	php_info_print_table_row(2, "Build Date", __DATE__);
	php_info_print_table_row(2, "Build Time", __TIME__);
	php_info_print_table_colspan_header(2, "Supported Features");
	strcpy(buf, PHP_CONSTANT_HWSIMSO0);
	strcat(buf, " ");
	strcat(buf, PHP_CONSTANT_HWSJAVA0);
	php_info_print_table_row(2, "Exit Modules", buf);
	strcpy(buf, PHP_CONSTANT_CP297);
	php_info_print_table_row(2, "ASCII / EBCDIC encodings", buf);
	php_info_print_table_end();
}

/**
 * Socket connection
 */
PHP_FUNCTION(ims_connect)
{
debug("--------------------------------------------------\n");
debug("ims_connect() -> START");

	char *			host;
	int				host_len;
	long			port = -1;
	char *			data;
	int				data_len;
	double			timeout = 30;
	unsigned long	conv;
	struct timeval	tv;
	php_stream *	stream = NULL;
	
	socket_connection_info	connection_info;

	/* PHP arguments recovery		*/
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl",
		&host,
		&host_len,
		&port) == FAILURE)
	{
		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Missing arguments");
		RETURN_NULL();
	}
	/* Timeout value conversion		*/
	conv = (unsigned long) (timeout * 1000000.0);
	tv.tv_sec	= conv / 1000000;
	tv.tv_usec	= conv % 1000000;
	
	/* Connection data information	*/
	connection_info.host			= host;
	connection_info.port			= port;
	connection_info.socktype		= SOCK_STREAM;
	connection_info.timeout			= &tv;
	connection_info.persistent_id	= NULL;
	
	/* Socket connection and stream recovery	*/
	if ((stream = socket_connect(&connection_info)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Connection failed %s:%ld", connection_info.host, connection_info.port);
		RETURN_NULL();
	}
	php_stream_to_zval(stream, return_value);

debug("ims_connect() -> END");
} 

/**
 * Message wrapping from parameters + XML format
 */
PHP_FUNCTION(ims_wrap_message)
{
debug("ims_wrap_message() -> START");

	/* Declarations : PHP function arguments	*/

	zval *					zParameters;		/* Parameters array			*/
	char *					fileName;			/* XML format file name		*/
	int						fileNameLen;
	char *					encoding;			/* Encoding to apply		*/
	int						encodingLen;

	/* Declarations : IMS treatment variables	*/

	binaryDataBuffer_t		dataBuffer;			/* Binary buffer			*/
	MSG_wrapper_t			msgWrapper;			/* Message wrapper			*/
	TRANS_trans_t *			trans;				/* Transcoding to use		*/
	IMS_field_t	*			field;				/* IMS message field		*/
	IMS_field_t	*			f_list;				/* IMS message fields list	*/
	zval **					zValue;				/* Current parameter value	*/
	int						result;				/* Function return code		*/
	
	/* PHP arguments recovery		*/

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ass",		/* :p	*/
		&zParameters,
		&fileName, &fileNameLen,
		&encoding, &encodingLen) == FAILURE)
	{
		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Missing arguments");
		RETURN_NULL();
	}

	/* Transcoding selection		*/
	
	if (TRANS_getByName(&trans, encoding) != TRANS_RESULT_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_ENCODING));
		RETURN_NULL();
	}

	/* XML parsing of message formating		*/

	if ((result = MSG_parseXML(fileName, &msgWrapper, trans)) != IMS_CODE_OK)
	{
/*	TODO : Wrong return code from parsing
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
		RETURN_NULL();
*/
	}
	f_list = msgWrapper.f_list;

debug("ims_wrap_message() -> End of XML parsing");
	/* IMS message generation		*/
	/* Fields list loop				*/

	if (binaryDataBuffer_initBuffer(&dataBuffer, msgWrapper.length) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_MALLOC));
		debug("ims_wrap_message() -> DataBuffer initialization failure");
	}
debug("ims_wrap_message() -> Message data generation");
	while (field = IMS_nextField(&f_list))
	{
debug("ims_wrap_message() -> IMS field writing");
debug(field->name);
		/* Use of parameter value if available	*/
		
		if (field->name && zend_hash_find(Z_ARRVAL_P(zParameters), field->name, strlen(field->name) + 1, (void**)&zValue) == FAILURE)
		{
			debug("ims_wrap_message() -> Parameter not found");
		}
		else
		{
			switch (Z_TYPE_P(*zValue))
			{
				case IS_NULL :		result = IMS_setField_char(field, IMS_ZERO);
									break;
				case IS_BOOL :		
				case IS_LONG :		result = IMS_setField_int(field, Z_LVAL_P(*zValue));
									break;
				case IS_STRING :	result = IMS_setField_str(field, Z_STRVAL_P(*zValue), Z_STRLEN_P(*zValue));
									break;
				case IS_DOUBLE :	
				case IS_RESOURCE :	
				case IS_ARRAY :		
				case IS_OBJECT :	
				default :
									debug("ims_wrap_message() -> Invalid parameter value");
			}
			if (result != IMS_CODE_OK)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
				debug("ims_wrap_message() -> Invalid parmeter compatibility");
			}
		}
		
		/* Writing og field data in buffer	*/
		
		if ((result = IMS_writeField(field, &dataBuffer)) != IMS_CODE_OK)
		{
			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s in field '%s'", IMS_getErrorLabel(result), field->name);
		}
	}
	
	/* Return of message data to PHP	*/

	if (result == IMS_CODE_OK)
	{
		RETVAL_STRINGL(dataBuffer.dataBuffer, dataBuffer.dataLength, 1);
	}
	else
	{
		RETVAL_NULL();
	}
debug("ims_wrap_message() -> END");
} 

/**
 * Transaction execution - Send / Receive data through IMS Connect
 */
PHP_FUNCTION(ims_execute)
{
debug("ims_execute() -> START");
	/* Declarations : PHP function arguments	*/
	
	zval *				zstream;				/* PHP stream variable		*/
	php_stream *		socketStream;			/* PHP socket stream		*/
	
	/* Declarations : IMS treatment variables	*/
	
	IMS_connect_t		connect;				/* Connecting parameters	*/
	IMS_message_t		message;				/* IMS message data			*/
	IMS_factory_t		factory;				/* IMS message manager		*/
	binaryDataBuffer_t	dataBuffer;				/* Binary data buffer		*/
	int					bufferLength;			/* Binary buffer length		*/
	int					result;					/* Function return code		*/

	/* Data initialization			 */

	IMS_initConnect(&connect);
	IMS_initMessage(&message);
	IMS_initFactory(&factory);

	/* PHP arguments recovery		*/
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rslssssssss",
		&zstream,
		&(connect.encoding),
		&(connect.encodingLen),
		&(connect.exitCode),
		&(connect.msgRecord),
		&(connect.msgRecordLen),
		&(connect.clientId),
		&(connect.clientIdLen),
		&(connect.tranCode),
		&(connect.tranCodeLen),
		&(connect.datastoreId),
		&(connect.datastoreIdLen),
		&(connect.lterm),
		&(connect.ltermLen),
		&(connect.racfUserId),
		&(connect.racfUserIdLen),
		&(connect.racfGroupName),
		&(connect.racfGroupNameLen),
		&(connect.racfPassword),
		&(connect.racfPasswordLen)) == FAILURE)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Impossible de rcuprer tous les arguments");
		RETURN_NULL();
	}
	php_stream_from_zval(socketStream, &zstream);

	/* IMS manager settings			*/
	
	if ((result = IMS_setFactory(&factory, &connect, &message)) != IMS_CODE_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
		RETURN_NULL();
	}
	
	/* IMS message settings			*/
	
	if ((result = IMS_buildMessage(&factory)) != IMS_CODE_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
		RETURN_NULL();
	}
	
	/* IMS message generation		*/
	
	if ((result = IMS_writeMessage(&factory, &dataBuffer)) != IMS_CODE_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
		RETURN_NULL();
	}
	
	/* IMS data sending to socket stream	*/
debug("ims_execute() -> Message sending");

	if (IMS_send(socketStream, &dataBuffer) != message.msgLength)
	{
		/* Sent length mismatching data	*/
		insertExecLog("OUTPUT ERROR", dataBuffer.dataBuffer, dataBuffer.dataLength);

		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_SEND));
		RETURN_NULL();
	}
	else
	{
		insertExecLog("OUTPUT", dataBuffer.dataBuffer, dataBuffer.dataLength);
	}
	binaryDataBuffer_freeBuffer(&dataBuffer);	/* Data buffer freeing	*/
	
debug("ims_execute() -> Message envoy");

	/* Response reception from socket stream	*/
	
	if (IMS_receive(socketStream, &dataBuffer) != dataBuffer.dataLength)
	{
		/* Received length mismatching data	*/
		insertExecLog("INPUT ERROR", dataBuffer.dataBuffer, dataBuffer.dataLength);
		
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_RECEIVE));
		RETVAL_NULL();
	}
	
	/* IMS message data reading		*/
	
debug("ims_execute() -> Message reading");
	if ((result = IMS_readMessage(&factory, &dataBuffer)) != IMS_CODE_OK)
	{
		/* Reception failure	*/
		insertExecLog("INPUT ERROR", dataBuffer.dataBuffer, dataBuffer.dataLength);
		
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
		RETVAL_NULL();
	}
	else
	{
		/* Return of response data to PHP */
		insertExecLog("INPUT", dataBuffer.dataBuffer, dataBuffer.dataLength);
		
		RETVAL_STRINGL(message.msgRecord, message.msgRecordLen, 0);
	}
	binaryDataBuffer_freeBuffer(&dataBuffer);	/* Data buffer freeing		*/
	socket_disconnect(socketStream);			/* Socket stream closing	*/

debug("ims_execute() -> END");
}

/**
 * Field values parsing from message data and XML formating
 */
PHP_FUNCTION(ims_unwrap_message)
{
debug("ims_unwrap_message() -> START");

	/* Declarations : PHP function arguments	*/

	char *					message;			/* Message to format		*/
	int						messageLen;
	char *					fileName;			/* XML format file name		*/
	int						fileNameLen;
	char *					encoding;			/* Encoding to apply		*/
	int						encodingLen;

	/* Declarations : IMS treatment variables	*/

	binaryDataBuffer_t		dataBuffer;			/* Binary data buffer		*/
	MSG_wrapper_t			msgWrapper;			/* Message wrapper			*/
	TRANS_trans_t *			trans;				/* Transcoding to use		*/
	IMS_field_t	*			field;				/* IMS message field		*/
	IMS_field_t	*			f_list;				/* IMS message fields list	*/
/*	zval **					zArray;				/* Parameters values array	*/
	int						result;				/* Function return code		*/
	
	/* PHP arguments recovery		*/

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sss",
		&message, &messageLen,
		&fileName, &fileNameLen,
		&encoding, &encodingLen) == FAILURE)
	{
		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Missing arguments");
		RETURN_NULL();
	}

	/* Transcoding selection		*/
	
	if (TRANS_getByName(&trans, encoding) != TRANS_RESULT_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_ENCODING));
		RETURN_NULL();
	}

	/* XML parsing of message formating	*/

	if ((result = MSG_parseXML(fileName, &msgWrapper, trans)) != IMS_CODE_OK)
	{
/*	TODO : Wrong return code from parsing
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
		RETURN_NULL();
*/
	}
	f_list = msgWrapper.f_list;

debug("ims_unwrap_message() -> End of XML parsing");

	/* Result array initialization		*/
	
	array_init(return_value);
	
	/* Binary buffer initialization		*/
	
	if (msgWrapper.length != messageLen)
	{
		/* Message length error		*/
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Format length [%d] and message length [%d] mismatch", msgWrapper.length, messageLen);
		debug("ims_unwrap_message() -> Error : Format and message length mismatch");
	}
	if (binaryDataBuffer_setBuffer(&dataBuffer, message, messageLen) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_MALLOC));
		debug("ims_unwrap_message() -> Error : Data buffer initialization failed");
	}
	/* Fields list loop		*/

debug("ims_unwrap_message() -> Result array values filling");

	while (field = IMS_nextField(&f_list))
	{
debug("ims_unwrap_message() -> IMS message field parsing");
debug(field->name);

		/* Data buffer insertion with field value	*/
		
		if ((result = IMS_readField(field, &dataBuffer)) != IMS_CODE_OK)
		{
			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s in field '%s'", IMS_getErrorLabel(result), field->name);
		}

		/* Array key setting from field name		*/

		switch (FIELD_TYPE(field))
		{
			case IMS_TYPE_NONE :	result = add_assoc_null(return_value, field->name);
									debug("ims_unwrap_message() -> NONE field type");
									break;
			case IMS_TYPE_CHAR :	result = add_assoc_long(return_value, field->name, FIELD_VCHR(field));
									debug("ims_unwrap_message() -> CHAR field type");
									break;
			case IMS_TYPE_SHORT :	result = add_assoc_long(return_value, field->name, FIELD_VSHR(field));
									debug("ims_unwrap_message() -> SHORT field type");
									break;
			case IMS_TYPE_INT :		result = add_assoc_long(return_value, field->name, FIELD_VINT(field));
									debug("ims_unwrap_message() -> INT field type");
									break;
			case IMS_TYPE_STRING :	result = add_assoc_stringl(return_value, field->name, FIELD_VSTR(field).string, FIELD_VSTR(field).length, 1);
									debug("ims_unwrap_message() -> STRING field type");
									break;
			default :				result = add_assoc_null(return_value, field->name);
									debug("ims_unwrap_message() -> Invalid field type");
		}
	}
	debug("ims_unwrap_message() -> END");
} 

/**
 * String encoding
 */
PHP_FUNCTION(ims_encode)
{
	/* Variables declarations		*/
	
	char *			srcStr		= NULL;		/* Source string to convert	*/
	char *			destStr		= NULL;		/* Converted result string	*/
	int				length		= 0;		/* String size				*/
	char *			encoding	= NULL;		/* Transcoding code			*/
	int				encodingLen	= 0;		/* Transcoding code length	*/
	TRANS_trans_t *	trans;					/* Transcoding to use		*/

	/* PHP arguments recovery		*/
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss",
		&srcStr,
		&length,
		&encoding,
		&encodingLen) == FAILURE)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Missing arguments");
		RETURN_STRINGL(srcStr, length, 0);
	}
	
	/* Transcoding selection		*/
	
	if (TRANS_getByName(&trans, encoding) != TRANS_RESULT_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_ENCODING));
		RETURN_STRINGL(srcStr, length, 0);
	}

	/* String encoding				*/
	
	if ((destStr = (char *) emalloc(length + 1)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_MALLOC));
		RETURN_STRINGL(srcStr, length, 0);
	}
	else
	{
		/* String memory copy	*/
		memcpy(destStr, srcStr, length);
		/* Final binary zero	*/
		destStr[length]	= 0x00;
	}
	if (TRANS_encode(destStr, length, trans) != TRANS_RESULT_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_TRANSCODE));
		RETURN_STRINGL(srcStr, length, 0);
	}
	else
	{
		RETURN_STRINGL(destStr, length, 0);
	}
}

/**
 * String decoding
 */
PHP_FUNCTION(ims_decode)
{
	/* Variables declarations		*/
	
	char *			srcStr		= NULL;		/* Source string to convert	*/
	char *			destStr		= NULL;		/* Converted result string	*/
	int				length		= 0;		/* String size				*/
	char *			encoding	= NULL;		/* Transcoding code			*/
	int				encodingLen	= 0;		/* Transcoding code length	*/
	TRANS_trans_t *	trans;					/* Transcoding to use		*/

	/* PHP arguments recovery		*/
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss",
		&srcStr,
		&length,
		&encoding,
		&encodingLen) == FAILURE)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Missing arguments");
		RETURN_STRINGL(srcStr, length, 0);
	}
	
	/* Transcoding selection			*/
	
	if (TRANS_getByName(&trans, encoding) != TRANS_RESULT_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_ENCODING));
		RETURN_STRINGL(srcStr, length, 0);
	}

	/* String decoding					*/
	
	if ((destStr = (char *) emalloc(length + 1)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_MALLOC));
		RETURN_STRINGL(srcStr, length, 0);
	}
	else
	{
		/* String memory copy	*/
		memcpy(destStr, srcStr, length);
		/* Final binary zero	*/
		destStr[length]	= 0x00;
	}
	if (TRANS_decode(destStr, length, trans) != TRANS_RESULT_OK)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(IMS_CODE_ERR_TRANSCODE));
		RETURN_STRINGL(srcStr, length, 0);
	}
	else
	{
		RETURN_STRINGL(destStr, length, 0);
	}
}

/**
 * String transcoding
 */
char *transcode(char *srcStr, int length, int encoding, char transWay)
{
	/* Variables declarations		*/
	
	char	encoder[TRANS_TBL_SIZE];	/* Encoding table			*/
	char	decoder[TRANS_TBL_SIZE];	/* Decoding table			*/
	char *	destStr	= NULL;				/* Converted result string	*/
	int		result	= IMS_CODE_OK;		/* Function return code		*/
	int		i		= 0;

	/* Transcoding selection			*/
/*
	switch (encoding) {
		case 0 :				break;
		case IMS_CODE_CP297 :	result = TRANSCODE_SET(cp297)(encoder, decoder);
								break;
		default :				result = IMS_CODE_ERR_ENCODING;
	}
	if (result != IMS_CODE_OK) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, IMS_getErrorLabel(result));
		return NULL;
	}
*/	
	/* String conversion				*/
	
	destStr = IMS_makeField(srcStr, length, length + 1, 0x00);

	if (destStr && encoding)
	{
		if (transWay == IMS_ENCODE)
		{
			for (i = 0; i < length; i++)
				destStr[i] = encoder[(unsigned char) destStr[i]];
		}
		if (transWay == IMS_DECODE)
		{
			for (i = 0; i < length; i++)
				destStr[i] = decoder[(unsigned char) destStr[i]];
		}
	}
	
	/* Converted string return		*/
	return destStr;
}

/**
 * Log insertion of IMS execution infos
 */
int	insertExecLog(const char *type, const char *data, int length)
{
	zval *	new_entry	= NULL;
	
	MAKE_STD_ZVAL(new_entry);
	if(array_init(new_entry) != SUCCESS)
	{
		debug("IMS_insertExecLog() -> Execution logging failure");
		return IMS_CODE_ERR;
	}
	if (add_assoc_string(new_entry, "type", type, 1) != SUCCESS)
	{
		return IMS_CODE_ERR;
	}
	if (add_assoc_stringl(new_entry, "data", data, length, 1) != SUCCESS)
	{
		return IMS_CODE_ERR;
	}
	if (add_next_index_zval(IMS_CONNECT_G(IMS_execLog), new_entry) != SUCCESS)
	{
		return IMS_CODE_ERR;
	}
	return IMS_CODE_OK;
}

/**
 * IMS execution log fetcher
 */
PHP_FUNCTION(ims_exec_log)
{
debug("ims_exec_log() -> START");
	*return_value = *IMS_CONNECT_G(IMS_execLog);
debug("ims_exec_log() -> END");
}

/**
 * Data hexadecimal display
 */
PHP_FUNCTION(ims_print_hexa)
{
	/* Variables declarations		*/
	
	char *			data		= NULL;		/* Donnes  afficher	*/
	int				dataLen		= 0;		/* Longueur des donnes	*/
	int				width		= 0;		/* Largeur d'affichage	*/

	/* PHP arguments recovery		*/
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl",
		&data,
		&dataLen,
		&width) == FAILURE)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Impossible de rcuprer tous les arguments");
		RETURN_NULL();
	}
	
	/* Data hexadecimal display 	*/	
	echoTrame(data, dataLen, width);
	
	RETURN_NULL();
}

/**
 * HTML Formated data display
 */
void echoTrame(char *data, int length, int width)
{
	int	i;				/* Current data offset		*/
	int	j;				/* Current line position	*/

	char	str[10];	/* HTML display buffer		*/
	
	/* Formated HTML display						*/
	php_printf("<PRE>");
	/* Data writing line by line					*/
	for (i = 0; i < length; i = j)
	{
		/* Offset display in current line			*/
		php_printf("%.4X |", i);

		/* Hexadecimal data display					*/
		for (j = i; j < length && (j - i) < width; j++)
		{
			/* 4 bytes groupes display	*/
			php_printf((j % 4 ? "%.2X" : " %.2X"), 0xff & data[j]);
		}
		/* Last line filler				*/
		while ((j - i) < width)
		{
			/* 4 bytes groupes display	*/
			php_printf(j % 4 ? "  " : "   ");
			j++;
		}

		/* ASCII data display in HTML				*/
		php_printf(" | ");
		for (j = i; j < length && (j - i) < width; j++)
		{
			/* HTML conversion						*/
			if (data[j] >= 0x20 && data[j] <= 0x7D)
			{
				/* Displayable characters			*/
				switch (data[j])
				{
					case 0x20 :	sprintf(str, "&nbsp;");
								break;
					case 0x22 :	sprintf(str, "&quot;");
								break;
					case 0x26 :	sprintf(str, "&amp;");
								break;
					case 0x27 :	sprintf(str, "&#039;");
								break;
					case 0x3C :	sprintf(str, "&lt;");
								break;
					case 0x3E :	sprintf(str, "&gt;");
								break;
					default :	sprintf(str, "%c", data[j]);
				}
			}
			else
			{
				/* Non displayable characters		*/
				sprintf(str, ".");
			}
			/* Current character HTML display		*/
			php_printf("%s", str);
		}
		/* Last line filler				*/
		while ((j - i) < width)
		{
			php_printf("&nbsp;");
			j++;
		}
		php_printf(" |\n");
	}
	/* End of HTML formated display					*/
	php_printf("</PRE>");
}
